/****************************************************************************
** Meta object code from reading C++ file 'SettingsDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/SettingsDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SettingsDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__FavoriteTagsPage_t {
    QByteArrayData data[10];
    char stringdata0[130];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__FavoriteTagsPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__FavoriteTagsPage_t qt_meta_stringdata_Gui__FavoriteTagsPage = {
    {
QT_MOC_LITERAL(0, 0, 21), // "Gui::FavoriteTagsPage"
QT_MOC_LITERAL(1, 22, 5), // "saved"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 14), // "widgetsUpdated"
QT_MOC_LITERAL(4, 44, 13), // "updateWidgets"
QT_MOC_LITERAL(5, 58, 9), // "moveTagBy"
QT_MOC_LITERAL(6, 68, 6), // "offset"
QT_MOC_LITERAL(7, 75, 16), // "addButtonClicked"
QT_MOC_LITERAL(8, 92, 17), // "editButtonClicked"
QT_MOC_LITERAL(9, 110, 19) // "deleteButtonClicked"

    },
    "Gui::FavoriteTagsPage\0saved\0\0"
    "widgetsUpdated\0updateWidgets\0moveTagBy\0"
    "offset\0addButtonClicked\0editButtonClicked\0"
    "deleteButtonClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__FavoriteTagsPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   51,    2, 0x08 /* Private */,
       5,    1,   52,    2, 0x08 /* Private */,
       7,    0,   55,    2, 0x08 /* Private */,
       8,    0,   56,    2, 0x08 /* Private */,
       9,    0,   57,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::FavoriteTagsPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FavoriteTagsPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saved(); break;
        case 1: _t->widgetsUpdated(); break;
        case 2: _t->updateWidgets(); break;
        case 3: _t->moveTagBy((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 4: _t->addButtonClicked(); break;
        case 5: _t->editButtonClicked(); break;
        case 6: _t->deleteButtonClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FavoriteTagsPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FavoriteTagsPage::saved)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FavoriteTagsPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FavoriteTagsPage::widgetsUpdated)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::FavoriteTagsPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_Gui__FavoriteTagsPage.data,
    qt_meta_data_Gui__FavoriteTagsPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::FavoriteTagsPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::FavoriteTagsPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__FavoriteTagsPage.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_FavoriteTagsPage"))
        return static_cast< Ui_FavoriteTagsPage*>(this);
    if (!strcmp(_clname, "ConfigurationWidgetInterface"))
        return static_cast< ConfigurationWidgetInterface*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int Gui::FavoriteTagsPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Gui::FavoriteTagsPage::saved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Gui::FavoriteTagsPage::widgetsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_Gui__GeneralPage_t {
    QByteArrayData data[12];
    char stringdata0[178];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__GeneralPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__GeneralPage_t qt_meta_stringdata_Gui__GeneralPage = {
    {
QT_MOC_LITERAL(0, 0, 16), // "Gui::GeneralPage"
QT_MOC_LITERAL(1, 17, 5), // "saved"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 15), // "reloadPasswords"
QT_MOC_LITERAL(4, 40, 14), // "widgetsUpdated"
QT_MOC_LITERAL(5, 55, 13), // "updateWidgets"
QT_MOC_LITERAL(6, 69, 14), // "moveIdentityUp"
QT_MOC_LITERAL(7, 84, 16), // "moveIdentityDown"
QT_MOC_LITERAL(8, 101, 16), // "addButtonClicked"
QT_MOC_LITERAL(9, 118, 17), // "editButtonClicked"
QT_MOC_LITERAL(10, 136, 19), // "deleteButtonClicked"
QT_MOC_LITERAL(11, 156, 21) // "passwordPluginChanged"

    },
    "Gui::GeneralPage\0saved\0\0reloadPasswords\0"
    "widgetsUpdated\0updateWidgets\0"
    "moveIdentityUp\0moveIdentityDown\0"
    "addButtonClicked\0editButtonClicked\0"
    "deleteButtonClicked\0passwordPluginChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__GeneralPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,
       3,    0,   65,    2, 0x06 /* Public */,
       4,    0,   66,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   67,    2, 0x08 /* Private */,
       6,    0,   68,    2, 0x08 /* Private */,
       7,    0,   69,    2, 0x08 /* Private */,
       8,    0,   70,    2, 0x08 /* Private */,
       9,    0,   71,    2, 0x08 /* Private */,
      10,    0,   72,    2, 0x08 /* Private */,
      11,    0,   73,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::GeneralPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GeneralPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saved(); break;
        case 1: _t->reloadPasswords(); break;
        case 2: _t->widgetsUpdated(); break;
        case 3: _t->updateWidgets(); break;
        case 4: _t->moveIdentityUp(); break;
        case 5: _t->moveIdentityDown(); break;
        case 6: _t->addButtonClicked(); break;
        case 7: _t->editButtonClicked(); break;
        case 8: _t->deleteButtonClicked(); break;
        case 9: _t->passwordPluginChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (GeneralPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GeneralPage::saved)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (GeneralPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GeneralPage::reloadPasswords)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (GeneralPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GeneralPage::widgetsUpdated)) {
                *result = 2;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::GeneralPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_Gui__GeneralPage.data,
    qt_meta_data_Gui__GeneralPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::GeneralPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::GeneralPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__GeneralPage.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_GeneralPage"))
        return static_cast< Ui_GeneralPage*>(this);
    if (!strcmp(_clname, "ConfigurationWidgetInterface"))
        return static_cast< ConfigurationWidgetInterface*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int Gui::GeneralPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Gui::GeneralPage::saved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Gui::GeneralPage::reloadPasswords()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Gui::GeneralPage::widgetsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
struct qt_meta_stringdata_Gui__EditIdentity_t {
    QByteArrayData data[6];
    char stringdata0[62];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__EditIdentity_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__EditIdentity_t qt_meta_stringdata_Gui__EditIdentity = {
    {
QT_MOC_LITERAL(0, 0, 17), // "Gui::EditIdentity"
QT_MOC_LITERAL(1, 18, 5), // "saved"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 14), // "widgetsUpdated"
QT_MOC_LITERAL(4, 40, 12), // "enableButton"
QT_MOC_LITERAL(5, 53, 8) // "onReject"

    },
    "Gui::EditIdentity\0saved\0\0widgetsUpdated\0"
    "enableButton\0onReject"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__EditIdentity[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x06 /* Public */,
       3,    0,   35,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   36,    2, 0x0a /* Public */,
       5,    0,   37,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::EditIdentity::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditIdentity *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saved(); break;
        case 1: _t->widgetsUpdated(); break;
        case 2: _t->enableButton(); break;
        case 3: _t->onReject(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EditIdentity::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditIdentity::saved)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (EditIdentity::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditIdentity::widgetsUpdated)) {
                *result = 1;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::EditIdentity::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Gui__EditIdentity.data,
    qt_meta_data_Gui__EditIdentity,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::EditIdentity::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::EditIdentity::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__EditIdentity.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_EditIdentity"))
        return static_cast< Ui_EditIdentity*>(this);
    return QDialog::qt_metacast(_clname);
}

int Gui::EditIdentity::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Gui::EditIdentity::saved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Gui::EditIdentity::widgetsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_Gui__EditFavoriteTag_t {
    QByteArrayData data[8];
    char stringdata0[94];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__EditFavoriteTag_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__EditFavoriteTag_t qt_meta_stringdata_Gui__EditFavoriteTag = {
    {
QT_MOC_LITERAL(0, 0, 20), // "Gui::EditFavoriteTag"
QT_MOC_LITERAL(1, 21, 5), // "saved"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 14), // "widgetsUpdated"
QT_MOC_LITERAL(4, 43, 15), // "tryEnableButton"
QT_MOC_LITERAL(5, 59, 8), // "onAccept"
QT_MOC_LITERAL(6, 68, 8), // "onReject"
QT_MOC_LITERAL(7, 77, 16) // "colorButtonClick"

    },
    "Gui::EditFavoriteTag\0saved\0\0widgetsUpdated\0"
    "tryEnableButton\0onAccept\0onReject\0"
    "colorButtonClick"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__EditFavoriteTag[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    0,   45,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   46,    2, 0x0a /* Public */,
       5,    0,   47,    2, 0x0a /* Public */,
       6,    0,   48,    2, 0x0a /* Public */,
       7,    0,   49,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::EditFavoriteTag::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditFavoriteTag *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saved(); break;
        case 1: _t->widgetsUpdated(); break;
        case 2: _t->tryEnableButton(); break;
        case 3: _t->onAccept(); break;
        case 4: _t->onReject(); break;
        case 5: _t->colorButtonClick(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EditFavoriteTag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditFavoriteTag::saved)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (EditFavoriteTag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditFavoriteTag::widgetsUpdated)) {
                *result = 1;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::EditFavoriteTag::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Gui__EditFavoriteTag.data,
    qt_meta_data_Gui__EditFavoriteTag,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::EditFavoriteTag::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::EditFavoriteTag::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__EditFavoriteTag.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_EditFavoriteTag"))
        return static_cast< Ui_EditFavoriteTag*>(this);
    return QDialog::qt_metacast(_clname);
}

int Gui::EditFavoriteTag::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Gui::EditFavoriteTag::saved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Gui::EditFavoriteTag::widgetsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_Gui__OutgoingPage_t {
    QByteArrayData data[11];
    char stringdata0[137];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__OutgoingPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__OutgoingPage_t qt_meta_stringdata_Gui__OutgoingPage = {
    {
QT_MOC_LITERAL(0, 0, 17), // "Gui::OutgoingPage"
QT_MOC_LITERAL(1, 18, 5), // "saved"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 14), // "widgetsUpdated"
QT_MOC_LITERAL(4, 40, 23), // "slotSetSubmissionMethod"
QT_MOC_LITERAL(5, 64, 13), // "updateWidgets"
QT_MOC_LITERAL(6, 78, 15), // "slotSetPassword"
QT_MOC_LITERAL(7, 94, 15), // "showPortWarning"
QT_MOC_LITERAL(8, 110, 7), // "warning"
QT_MOC_LITERAL(9, 118, 13), // "setPortByText"
QT_MOC_LITERAL(10, 132, 4) // "text"

    },
    "Gui::OutgoingPage\0saved\0\0widgetsUpdated\0"
    "slotSetSubmissionMethod\0updateWidgets\0"
    "slotSetPassword\0showPortWarning\0warning\0"
    "setPortByText\0text"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__OutgoingPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   51,    2, 0x08 /* Private */,
       5,    0,   52,    2, 0x08 /* Private */,
       6,    0,   53,    2, 0x08 /* Private */,
       7,    1,   54,    2, 0x08 /* Private */,
       9,    1,   57,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,   10,

       0        // eod
};

void Gui::OutgoingPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OutgoingPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saved(); break;
        case 1: _t->widgetsUpdated(); break;
        case 2: _t->slotSetSubmissionMethod(); break;
        case 3: _t->updateWidgets(); break;
        case 4: _t->slotSetPassword(); break;
        case 5: _t->showPortWarning((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->setPortByText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (OutgoingPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OutgoingPage::saved)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (OutgoingPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OutgoingPage::widgetsUpdated)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::OutgoingPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_Gui__OutgoingPage.data,
    qt_meta_data_Gui__OutgoingPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::OutgoingPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::OutgoingPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__OutgoingPage.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_OutgoingPage"))
        return static_cast< Ui_OutgoingPage*>(this);
    if (!strcmp(_clname, "ConfigurationWidgetInterface"))
        return static_cast< ConfigurationWidgetInterface*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int Gui::OutgoingPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Gui::OutgoingPage::saved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Gui::OutgoingPage::widgetsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_Gui__ImapPage_t {
    QByteArrayData data[8];
    char stringdata0[98];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__ImapPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__ImapPage_t qt_meta_stringdata_Gui__ImapPage = {
    {
QT_MOC_LITERAL(0, 0, 13), // "Gui::ImapPage"
QT_MOC_LITERAL(1, 14, 5), // "saved"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 14), // "widgetsUpdated"
QT_MOC_LITERAL(4, 36, 13), // "updateWidgets"
QT_MOC_LITERAL(5, 50, 20), // "maybeShowPortWarning"
QT_MOC_LITERAL(6, 71, 10), // "changePort"
QT_MOC_LITERAL(7, 82, 15) // "slotSetPassword"

    },
    "Gui::ImapPage\0saved\0\0widgetsUpdated\0"
    "updateWidgets\0maybeShowPortWarning\0"
    "changePort\0slotSetPassword"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__ImapPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    0,   45,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   46,    2, 0x08 /* Private */,
       5,    0,   47,    2, 0x08 /* Private */,
       6,    0,   48,    2, 0x08 /* Private */,
       7,    0,   49,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::ImapPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ImapPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saved(); break;
        case 1: _t->widgetsUpdated(); break;
        case 2: _t->updateWidgets(); break;
        case 3: _t->maybeShowPortWarning(); break;
        case 4: _t->changePort(); break;
        case 5: _t->slotSetPassword(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ImapPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapPage::saved)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ImapPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapPage::widgetsUpdated)) {
                *result = 1;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::ImapPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_Gui__ImapPage.data,
    qt_meta_data_Gui__ImapPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::ImapPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::ImapPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__ImapPage.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_ImapPage"))
        return static_cast< Ui_ImapPage*>(this);
    if (!strcmp(_clname, "ConfigurationWidgetInterface"))
        return static_cast< ConfigurationWidgetInterface*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int Gui::ImapPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Gui::ImapPage::saved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Gui::ImapPage::widgetsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_Gui__CachePage_t {
    QByteArrayData data[5];
    char stringdata0[51];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__CachePage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__CachePage_t qt_meta_stringdata_Gui__CachePage = {
    {
QT_MOC_LITERAL(0, 0, 14), // "Gui::CachePage"
QT_MOC_LITERAL(1, 15, 5), // "saved"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 14), // "widgetsUpdated"
QT_MOC_LITERAL(4, 37, 13) // "updateWidgets"

    },
    "Gui::CachePage\0saved\0\0widgetsUpdated\0"
    "updateWidgets"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__CachePage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,
       3,    0,   30,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   31,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void Gui::CachePage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CachePage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saved(); break;
        case 1: _t->widgetsUpdated(); break;
        case 2: _t->updateWidgets(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CachePage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CachePage::saved)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CachePage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CachePage::widgetsUpdated)) {
                *result = 1;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::CachePage::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_Gui__CachePage.data,
    qt_meta_data_Gui__CachePage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::CachePage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::CachePage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__CachePage.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_CachePage"))
        return static_cast< Ui_CachePage*>(this);
    if (!strcmp(_clname, "ConfigurationWidgetInterface"))
        return static_cast< ConfigurationWidgetInterface*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int Gui::CachePage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Gui::CachePage::saved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Gui::CachePage::widgetsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_Gui__SettingsDialog_t {
    QByteArrayData data[7];
    char stringdata0[95];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__SettingsDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__SettingsDialog_t qt_meta_stringdata_Gui__SettingsDialog = {
    {
QT_MOC_LITERAL(0, 0, 19), // "Gui::SettingsDialog"
QT_MOC_LITERAL(1, 20, 24), // "reloadPasswordsRequested"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 6), // "accept"
QT_MOC_LITERAL(4, 53, 6), // "reject"
QT_MOC_LITERAL(5, 60, 23), // "adjustSizeToScrollAreas"
QT_MOC_LITERAL(6, 84, 10) // "slotAccept"

    },
    "Gui::SettingsDialog\0reloadPasswordsRequested\0"
    "\0accept\0reject\0adjustSizeToScrollAreas\0"
    "slotAccept"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__SettingsDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   40,    2, 0x0a /* Public */,
       4,    0,   41,    2, 0x0a /* Public */,
       5,    0,   42,    2, 0x08 /* Private */,
       6,    0,   43,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::SettingsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SettingsDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->reloadPasswordsRequested(); break;
        case 1: _t->accept(); break;
        case 2: _t->reject(); break;
        case 3: _t->adjustSizeToScrollAreas(); break;
        case 4: _t->slotAccept(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SettingsDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SettingsDialog::reloadPasswordsRequested)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::SettingsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Gui__SettingsDialog.data,
    qt_meta_data_Gui__SettingsDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::SettingsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::SettingsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__SettingsDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Gui::SettingsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void Gui::SettingsDialog::reloadPasswordsRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
