/********************************************************************************
** Form generated from reading UI file 'SettingsImapPage.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGSIMAPPAGE_H
#define UI_SETTINGSIMAPPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include "LineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_ImapPage
{
public:
    QWidget *scrollAreaWidgetContents;
    QFormLayout *formLayout;
    QLabel *methodLabel;
    QComboBox *method;
    QLabel *label;
    QComboBox *encryption;
    QLabel *imapHostLabel;
    LineEdit *imapHost;
    QLabel *imapPortLabel;
    LineEdit *imapPort;
    QLabel *portWarning;
    QLabel *imapUseSystemProxyLabel;
    QCheckBox *imapUseSystemProxy;
    QLabel *imapNeedsNetworkLabel;
    QCheckBox *imapNeedsNetwork;
    QLabel *imapUserLabel;
    LineEdit *imapUser;
    QLabel *imapPassLabel;
    LineEdit *imapPass;
    QLabel *passwordWarning;
    QLabel *passwordPluginStatus;
    QLabel *processPathLabel;
    LineEdit *processPath;
    QLabel *imapCapabilitiesBlacklistLabel;
    LineEdit *imapCapabilitiesBlacklist;
    QLabel *imapIdleRenewalLabel;
    QSpinBox *imapIdleRenewal;
    QLabel *imapNumberRefreshIntervalLabel;
    QSpinBox *imapNumberRefreshInterval;
    QLabel *accountIconLabel;
    QLineEdit *accountIcon;
    QLabel *archiveFolderLabel;
    LineEdit *archiveFolderName;

    void setupUi(QScrollArea *ImapPage)
    {
        if (ImapPage->objectName().isEmpty())
            ImapPage->setObjectName(QString::fromUtf8("ImapPage"));
        ImapPage->setFrameShadow(QFrame::Plain);
        ImapPage->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 237, 477));
        formLayout = new QFormLayout(scrollAreaWidgetContents);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        methodLabel = new QLabel(scrollAreaWidgetContents);
        methodLabel->setObjectName(QString::fromUtf8("methodLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, methodLabel);

        method = new QComboBox(scrollAreaWidgetContents);
        method->setObjectName(QString::fromUtf8("method"));

        formLayout->setWidget(0, QFormLayout::FieldRole, method);

        label = new QLabel(scrollAreaWidgetContents);
        label->setObjectName(QString::fromUtf8("label"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label);

        encryption = new QComboBox(scrollAreaWidgetContents);
        encryption->setObjectName(QString::fromUtf8("encryption"));

        formLayout->setWidget(1, QFormLayout::FieldRole, encryption);

        imapHostLabel = new QLabel(scrollAreaWidgetContents);
        imapHostLabel->setObjectName(QString::fromUtf8("imapHostLabel"));

        formLayout->setWidget(2, QFormLayout::LabelRole, imapHostLabel);

        imapHost = new LineEdit(scrollAreaWidgetContents);
        imapHost->setObjectName(QString::fromUtf8("imapHost"));

        formLayout->setWidget(2, QFormLayout::FieldRole, imapHost);

        imapPortLabel = new QLabel(scrollAreaWidgetContents);
        imapPortLabel->setObjectName(QString::fromUtf8("imapPortLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, imapPortLabel);

        imapPort = new LineEdit(scrollAreaWidgetContents);
        imapPort->setObjectName(QString::fromUtf8("imapPort"));

        formLayout->setWidget(3, QFormLayout::FieldRole, imapPort);

        portWarning = new QLabel(scrollAreaWidgetContents);
        portWarning->setObjectName(QString::fromUtf8("portWarning"));
        portWarning->setText(QString::fromUtf8(""));
        portWarning->setWordWrap(true);

        formLayout->setWidget(4, QFormLayout::SpanningRole, portWarning);

        imapUseSystemProxyLabel = new QLabel(scrollAreaWidgetContents);
        imapUseSystemProxyLabel->setObjectName(QString::fromUtf8("imapUseSystemProxyLabel"));

        formLayout->setWidget(5, QFormLayout::LabelRole, imapUseSystemProxyLabel);

        imapUseSystemProxy = new QCheckBox(scrollAreaWidgetContents);
        imapUseSystemProxy->setObjectName(QString::fromUtf8("imapUseSystemProxy"));

        formLayout->setWidget(5, QFormLayout::FieldRole, imapUseSystemProxy);

        imapNeedsNetworkLabel = new QLabel(scrollAreaWidgetContents);
        imapNeedsNetworkLabel->setObjectName(QString::fromUtf8("imapNeedsNetworkLabel"));

        formLayout->setWidget(6, QFormLayout::LabelRole, imapNeedsNetworkLabel);

        imapNeedsNetwork = new QCheckBox(scrollAreaWidgetContents);
        imapNeedsNetwork->setObjectName(QString::fromUtf8("imapNeedsNetwork"));

        formLayout->setWidget(6, QFormLayout::FieldRole, imapNeedsNetwork);

        imapUserLabel = new QLabel(scrollAreaWidgetContents);
        imapUserLabel->setObjectName(QString::fromUtf8("imapUserLabel"));

        formLayout->setWidget(7, QFormLayout::LabelRole, imapUserLabel);

        imapUser = new LineEdit(scrollAreaWidgetContents);
        imapUser->setObjectName(QString::fromUtf8("imapUser"));

        formLayout->setWidget(7, QFormLayout::FieldRole, imapUser);

        imapPassLabel = new QLabel(scrollAreaWidgetContents);
        imapPassLabel->setObjectName(QString::fromUtf8("imapPassLabel"));

        formLayout->setWidget(8, QFormLayout::LabelRole, imapPassLabel);

        imapPass = new LineEdit(scrollAreaWidgetContents);
        imapPass->setObjectName(QString::fromUtf8("imapPass"));
        imapPass->setEchoMode(QLineEdit::Password);

        formLayout->setWidget(8, QFormLayout::FieldRole, imapPass);

        passwordWarning = new QLabel(scrollAreaWidgetContents);
        passwordWarning->setObjectName(QString::fromUtf8("passwordWarning"));
        passwordWarning->setWordWrap(true);

        formLayout->setWidget(9, QFormLayout::SpanningRole, passwordWarning);

        passwordPluginStatus = new QLabel(scrollAreaWidgetContents);
        passwordPluginStatus->setObjectName(QString::fromUtf8("passwordPluginStatus"));
        passwordPluginStatus->setText(QString::fromUtf8("plugin status"));
        passwordPluginStatus->setAlignment(Qt::AlignCenter);
        passwordPluginStatus->setWordWrap(true);
        passwordPluginStatus->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout->setWidget(10, QFormLayout::SpanningRole, passwordPluginStatus);

        processPathLabel = new QLabel(scrollAreaWidgetContents);
        processPathLabel->setObjectName(QString::fromUtf8("processPathLabel"));

        formLayout->setWidget(11, QFormLayout::LabelRole, processPathLabel);

        processPath = new LineEdit(scrollAreaWidgetContents);
        processPath->setObjectName(QString::fromUtf8("processPath"));

        formLayout->setWidget(11, QFormLayout::FieldRole, processPath);

        imapCapabilitiesBlacklistLabel = new QLabel(scrollAreaWidgetContents);
        imapCapabilitiesBlacklistLabel->setObjectName(QString::fromUtf8("imapCapabilitiesBlacklistLabel"));

        formLayout->setWidget(12, QFormLayout::LabelRole, imapCapabilitiesBlacklistLabel);

        imapCapabilitiesBlacklist = new LineEdit(scrollAreaWidgetContents);
        imapCapabilitiesBlacklist->setObjectName(QString::fromUtf8("imapCapabilitiesBlacklist"));

        formLayout->setWidget(12, QFormLayout::FieldRole, imapCapabilitiesBlacklist);

        imapIdleRenewalLabel = new QLabel(scrollAreaWidgetContents);
        imapIdleRenewalLabel->setObjectName(QString::fromUtf8("imapIdleRenewalLabel"));

        formLayout->setWidget(13, QFormLayout::LabelRole, imapIdleRenewalLabel);

        imapIdleRenewal = new QSpinBox(scrollAreaWidgetContents);
        imapIdleRenewal->setObjectName(QString::fromUtf8("imapIdleRenewal"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(imapIdleRenewal->sizePolicy().hasHeightForWidth());
        imapIdleRenewal->setSizePolicy(sizePolicy);
        imapIdleRenewal->setMinimum(1);
        imapIdleRenewal->setMaximum(29);

        formLayout->setWidget(13, QFormLayout::FieldRole, imapIdleRenewal);

        imapNumberRefreshIntervalLabel = new QLabel(scrollAreaWidgetContents);
        imapNumberRefreshIntervalLabel->setObjectName(QString::fromUtf8("imapNumberRefreshIntervalLabel"));

        formLayout->setWidget(15, QFormLayout::LabelRole, imapNumberRefreshIntervalLabel);

        imapNumberRefreshInterval = new QSpinBox(scrollAreaWidgetContents);
        imapNumberRefreshInterval->setObjectName(QString::fromUtf8("imapNumberRefreshInterval"));
        imapNumberRefreshInterval->setMinimum(30);
        imapNumberRefreshInterval->setMaximum(1740);
        imapNumberRefreshInterval->setValue(300);

        formLayout->setWidget(15, QFormLayout::FieldRole, imapNumberRefreshInterval);

        accountIconLabel = new QLabel(scrollAreaWidgetContents);
        accountIconLabel->setObjectName(QString::fromUtf8("accountIconLabel"));

        formLayout->setWidget(16, QFormLayout::LabelRole, accountIconLabel);

        accountIcon = new QLineEdit(scrollAreaWidgetContents);
        accountIcon->setObjectName(QString::fromUtf8("accountIcon"));
        accountIcon->setClearButtonEnabled(true);

        formLayout->setWidget(16, QFormLayout::FieldRole, accountIcon);

        archiveFolderLabel = new QLabel(scrollAreaWidgetContents);
        archiveFolderLabel->setObjectName(QString::fromUtf8("archiveFolderLabel"));

        formLayout->setWidget(17, QFormLayout::LabelRole, archiveFolderLabel);

        archiveFolderName = new LineEdit(scrollAreaWidgetContents);
        archiveFolderName->setObjectName(QString::fromUtf8("archiveFolderName"));

        formLayout->setWidget(17, QFormLayout::FieldRole, archiveFolderName);

        ImapPage->setWidget(scrollAreaWidgetContents);
#if QT_CONFIG(shortcut)
        methodLabel->setBuddy(method);
        imapHostLabel->setBuddy(imapHost);
        imapPortLabel->setBuddy(imapPort);
        imapUseSystemProxyLabel->setBuddy(imapUseSystemProxy);
        imapNeedsNetworkLabel->setBuddy(imapNeedsNetwork);
        imapUserLabel->setBuddy(imapUser);
        imapPassLabel->setBuddy(imapPass);
        processPathLabel->setBuddy(processPath);
        imapCapabilitiesBlacklistLabel->setBuddy(imapCapabilitiesBlacklist);
        archiveFolderLabel->setBuddy(archiveFolderName);
#endif // QT_CONFIG(shortcut)

        retranslateUi(ImapPage);

        QMetaObject::connectSlotsByName(ImapPage);
    } // setupUi

    void retranslateUi(QScrollArea *ImapPage)
    {
        methodLabel->setText(QCoreApplication::translate("ImapPage", "M&ethod", nullptr));
#if QT_CONFIG(tooltip)
        method->setToolTip(QCoreApplication::translate("ImapPage", "How to connect to the IMAP server", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        method->setWhatsThis(QCoreApplication::translate("ImapPage", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'DejaVu Sans'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Choose <span style=\" font-style:italic;\">TCP</span> if you want to connect via standard, non-encrypted connection and negotiate encryption transparently before login.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Choose <span style=\" font-style:italic;\">SSL</span> to connect over an encrypted socket from the very beginning"
                        " of the connection.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Finally, pick the <span style=\" font-style:italic;\">Local process</span> if you want to execute a process which then either connects to the IMAP server, or acts as one. This option enables accessing your IMAP server via SSH.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        label->setText(QCoreApplication::translate("ImapPage", "Encryption", nullptr));
        imapHostLabel->setText(QCoreApplication::translate("ImapPage", "Ser&ver", nullptr));
#if QT_CONFIG(tooltip)
        imapHost->setToolTip(QCoreApplication::translate("ImapPage", "Hostname of the IMAP server", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        imapHost->setWhatsThis(QCoreApplication::translate("ImapPage", "This is the name of the IMAP server that Trojit\303\241 should connect to.", nullptr));
#endif // QT_CONFIG(whatsthis)
        imapHost->setText(QString());
        imapPortLabel->setText(QCoreApplication::translate("ImapPage", "&Port", nullptr));
#if QT_CONFIG(tooltip)
        imapPort->setToolTip(QCoreApplication::translate("ImapPage", "Port number of the IMAP server", nullptr));
#endif // QT_CONFIG(tooltip)
        imapPort->setText(QString());
        imapUseSystemProxyLabel->setText(QCoreApplication::translate("ImapPage", "Use System Proxy Settings", nullptr));
#if QT_CONFIG(tooltip)
        imapNeedsNetworkLabel->setToolTip(QCoreApplication::translate("ImapPage", "Check if Trojit\303\241 should connect to a network in order to access this account. You will stay offline if there is no network connectivity.", nullptr));
#endif // QT_CONFIG(tooltip)
        imapNeedsNetworkLabel->setText(QCoreApplication::translate("ImapPage", "Needs Network", nullptr));
        imapUserLabel->setText(QCoreApplication::translate("ImapPage", "&Username", nullptr));
#if QT_CONFIG(tooltip)
        imapUser->setToolTip(QCoreApplication::translate("ImapPage", "Username", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        imapUser->setWhatsThis(QCoreApplication::translate("ImapPage", "User name to log in to the IMAP server", nullptr));
#endif // QT_CONFIG(whatsthis)
        imapUser->setText(QString());
        imapPassLabel->setText(QCoreApplication::translate("ImapPage", "Pass&word", nullptr));
#if QT_CONFIG(tooltip)
        imapPass->setToolTip(QCoreApplication::translate("ImapPage", "Password", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        imapPass->setWhatsThis(QCoreApplication::translate("ImapPage", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'DejaVu Sans'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Password to use when connecting to the IMAP server.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This password will be saved in <span style=\" font-weight:600;\">clear text</span> on this device, without any encryption. Unless you use a very safe form of a full disk encryption, it is a <span style=\" font-weight:600;\">bad id"
                        "ea</span> to save your password here.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">If you don't save the password, Trojit\303\241 will prompt for one when it needs it, but only once per session.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        imapPass->setText(QString());
        passwordWarning->setText(QCoreApplication::translate("ImapPage", "The password will be stored in plaintext. Leave it blank for Trojit\303\241 to prompt when needed.", nullptr));
        processPathLabel->setText(QCoreApplication::translate("ImapPage", "Path to Server &Binary", nullptr));
#if QT_CONFIG(whatsthis)
        processPath->setWhatsThis(QCoreApplication::translate("ImapPage", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'DejaVu Sans'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">If you choose the <span style=\" font-style:italic;\">Local process</span> option to connect to the IMAP server, this field specifies the program to start. For example, to connect to a remote mail server via SSH and launch a Dovecot instance in there, you could use the following:</p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Courier New,courier';\">ssh mail.example.org dovecot --exec-mail imap</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom"
                        ":0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Note that this command has to be absolutely non-interactive, and therefore you have to have your SSH set up to login without password, perhaps via the SSH keys.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        processPath->setText(QString());
        imapCapabilitiesBlacklistLabel->setText(QCoreApplication::translate("ImapPage", "Extension &Blacklist", nullptr));
        imapIdleRenewalLabel->setText(QCoreApplication::translate("ImapPage", "Timeout of IMAP IDLE", nullptr));
#if QT_CONFIG(tooltip)
        imapIdleRenewal->setToolTip(QCoreApplication::translate("ImapPage", "How often to renew the IDLE session.\n"
"This is useful to prevent timeouts due to broken or overly aggressive network middleboxes,\n"
"NAT gateways and other nasty annoyances.\n"
"The default value is 29 minutes.", nullptr));
#endif // QT_CONFIG(tooltip)
        imapNumberRefreshIntervalLabel->setText(QCoreApplication::translate("ImapPage", "Refresh Interval", nullptr));
#if QT_CONFIG(tooltip)
        imapNumberRefreshInterval->setToolTip(QCoreApplication::translate("ImapPage", "How often to check for new emails in all IMAP folders.\n"
"The default value is 300 seconds (5 minutes).", nullptr));
#endif // QT_CONFIG(tooltip)
        imapNumberRefreshInterval->setSuffix(QCoreApplication::translate("ImapPage", " seconds", nullptr));
        accountIconLabel->setText(QCoreApplication::translate("ImapPage", "Account Icon", nullptr));
        accountIcon->setPlaceholderText(QCoreApplication::translate("ImapPage", "Default Trojit\303\241 icon", nullptr));
        archiveFolderLabel->setText(QCoreApplication::translate("ImapPage", "Archive Folder &Name", nullptr));
        (void)ImapPage;
    } // retranslateUi

};

namespace Ui {
    class ImapPage: public Ui_ImapPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGSIMAPPAGE_H
