/********************************************************************************
** Form generated from reading UI file 'TagAddDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TAGADDDIALOG_H
#define UI_TAGADDDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_TagAddDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *label_1;
    QLineEdit *lineEdit;
    QLabel *tagsLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TagAddDialog)
    {
        if (TagAddDialog->objectName().isEmpty())
            TagAddDialog->setObjectName(QString::fromUtf8("TagAddDialog"));
        TagAddDialog->resize(320, 144);
        verticalLayout_2 = new QVBoxLayout(TagAddDialog);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        label_1 = new QLabel(TagAddDialog);
        label_1->setObjectName(QString::fromUtf8("label_1"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_1->sizePolicy().hasHeightForWidth());
        label_1->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(label_1);

        lineEdit = new QLineEdit(TagAddDialog);
        lineEdit->setObjectName(QString::fromUtf8("lineEdit"));

        verticalLayout_2->addWidget(lineEdit);

        tagsLabel = new QLabel(TagAddDialog);
        tagsLabel->setObjectName(QString::fromUtf8("tagsLabel"));
        tagsLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        verticalLayout_2->addWidget(tagsLabel);

        buttonBox = new QDialogButtonBox(TagAddDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(TagAddDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), TagAddDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), TagAddDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(TagAddDialog);
    } // setupUi

    void retranslateUi(QDialog *TagAddDialog)
    {
        TagAddDialog->setWindowTitle(QCoreApplication::translate("TagAddDialog", "Add Tags", nullptr));
        label_1->setText(QCoreApplication::translate("TagAddDialog", "Tag names (space-separated):", nullptr));
        tagsLabel->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class TagAddDialog: public Ui_TagAddDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TAGADDDIALOG_H
