/****************************************************************************
** Meta object code from reading C++ file 'Formatting.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/UiUtils/Formatting.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Formatting.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_UiUtils__Formatting_t {
    QByteArrayData data[14];
    char stringdata0[155];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UiUtils__Formatting_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UiUtils__Formatting_t qt_meta_stringdata_UiUtils__Formatting = {
    {
QT_MOC_LITERAL(0, 0, 19), // "UiUtils::Formatting"
QT_MOC_LITERAL(1, 20, 10), // "prettySize"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 5), // "bytes"
QT_MOC_LITERAL(4, 38, 10), // "prettyDate"
QT_MOC_LITERAL(5, 49, 8), // "dateTime"
QT_MOC_LITERAL(6, 58, 16), // "htmlizedTextPart"
QT_MOC_LITERAL(7, 75, 11), // "QModelIndex"
QT_MOC_LITERAL(8, 87, 9), // "partIndex"
QT_MOC_LITERAL(9, 97, 4), // "font"
QT_MOC_LITERAL(10, 102, 15), // "backgroundColor"
QT_MOC_LITERAL(11, 118, 9), // "textColor"
QT_MOC_LITERAL(12, 128, 9), // "linkColor"
QT_MOC_LITERAL(13, 138, 16) // "visitedLinkColor"

    },
    "UiUtils::Formatting\0prettySize\0\0bytes\0"
    "prettyDate\0dateTime\0htmlizedTextPart\0"
    "QModelIndex\0partIndex\0font\0backgroundColor\0"
    "textColor\0linkColor\0visitedLinkColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UiUtils__Formatting[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // methods: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x02 /* Public */,
       4,    1,   32,    2, 0x02 /* Public */,
       6,    6,   35,    2, 0x02 /* Public */,

 // methods: parameters
    QMetaType::QString, QMetaType::ULongLong,    3,
    QMetaType::QString, QMetaType::QDateTime,    5,
    QMetaType::QString, 0x80000000 | 7, QMetaType::QFont, QMetaType::QColor, QMetaType::QColor, QMetaType::QColor, QMetaType::QColor,    8,    9,   10,   11,   12,   13,

       0        // eod
};

void UiUtils::Formatting::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Formatting *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QString _r = _t->prettySize((*reinterpret_cast< quint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->prettyDate((*reinterpret_cast< const QDateTime(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->htmlizedTextPart((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QFont(*)>(_a[2])),(*reinterpret_cast< const QColor(*)>(_a[3])),(*reinterpret_cast< const QColor(*)>(_a[4])),(*reinterpret_cast< const QColor(*)>(_a[5])),(*reinterpret_cast< const QColor(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UiUtils::Formatting::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_UiUtils__Formatting.data,
    qt_meta_data_UiUtils__Formatting,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UiUtils::Formatting::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UiUtils::Formatting::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UiUtils__Formatting.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int UiUtils::Formatting::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
