/****************************************************************************
** Meta object code from reading C++ file 'qwwsmtpclient.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/qwwsmtpclient/qwwsmtpclient.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwwsmtpclient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QwwSmtpClient_t {
    QByteArrayData data[53];
    char stringdata0[606];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QwwSmtpClient_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QwwSmtpClient_t qt_meta_stringdata_QwwSmtpClient = {
    {
QT_MOC_LITERAL(0, 0, 13), // "QwwSmtpClient"
QT_MOC_LITERAL(1, 14, 4), // "done"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 9), // "connected"
QT_MOC_LITERAL(4, 30, 12), // "disconnected"
QT_MOC_LITERAL(5, 43, 12), // "stateChanged"
QT_MOC_LITERAL(6, 56, 5), // "State"
QT_MOC_LITERAL(7, 62, 15), // "commandFinished"
QT_MOC_LITERAL(8, 78, 5), // "error"
QT_MOC_LITERAL(9, 84, 14), // "commandStarted"
QT_MOC_LITERAL(10, 99, 10), // "tlsStarted"
QT_MOC_LITERAL(11, 110, 13), // "authenticated"
QT_MOC_LITERAL(12, 124, 15), // "rawCommandReply"
QT_MOC_LITERAL(13, 140, 4), // "code"
QT_MOC_LITERAL(14, 145, 7), // "details"
QT_MOC_LITERAL(15, 153, 9), // "sslErrors"
QT_MOC_LITERAL(16, 163, 16), // "QList<QSslError>"
QT_MOC_LITERAL(17, 180, 11), // "socketError"
QT_MOC_LITERAL(18, 192, 28), // "QAbstractSocket::SocketError"
QT_MOC_LITERAL(19, 221, 3), // "err"
QT_MOC_LITERAL(20, 225, 7), // "message"
QT_MOC_LITERAL(21, 233, 11), // "logReceived"
QT_MOC_LITERAL(22, 245, 4), // "data"
QT_MOC_LITERAL(23, 250, 7), // "logSent"
QT_MOC_LITERAL(24, 258, 18), // "disconnectFromHost"
QT_MOC_LITERAL(25, 277, 8), // "startTls"
QT_MOC_LITERAL(26, 286, 15), // "ignoreSslErrors"
QT_MOC_LITERAL(27, 302, 11), // "onConnected"
QT_MOC_LITERAL(28, 314, 14), // "onDisconnected"
QT_MOC_LITERAL(29, 329, 7), // "onError"
QT_MOC_LITERAL(30, 337, 17), // "_q_readFromSocket"
QT_MOC_LITERAL(31, 355, 12), // "_q_encrypted"
QT_MOC_LITERAL(32, 368, 12), // "Disconnected"
QT_MOC_LITERAL(33, 381, 10), // "Connecting"
QT_MOC_LITERAL(34, 392, 9), // "Connected"
QT_MOC_LITERAL(35, 402, 12), // "TLSRequested"
QT_MOC_LITERAL(36, 415, 14), // "Authenticating"
QT_MOC_LITERAL(37, 430, 7), // "Sending"
QT_MOC_LITERAL(38, 438, 13), // "Disconnecting"
QT_MOC_LITERAL(39, 452, 7), // "Options"
QT_MOC_LITERAL(40, 460, 6), // "Option"
QT_MOC_LITERAL(41, 467, 9), // "NoOptions"
QT_MOC_LITERAL(42, 477, 14), // "StartTlsOption"
QT_MOC_LITERAL(43, 492, 10), // "SizeOption"
QT_MOC_LITERAL(44, 503, 16), // "PipeliningOption"
QT_MOC_LITERAL(45, 520, 18), // "EightBitMimeOption"
QT_MOC_LITERAL(46, 539, 10), // "AuthOption"
QT_MOC_LITERAL(47, 550, 8), // "AuthMode"
QT_MOC_LITERAL(48, 559, 8), // "AuthNone"
QT_MOC_LITERAL(49, 568, 7), // "AuthAny"
QT_MOC_LITERAL(50, 576, 9), // "AuthPlain"
QT_MOC_LITERAL(51, 586, 9), // "AuthLogin"
QT_MOC_LITERAL(52, 596, 9) // "AuthModes"

    },
    "QwwSmtpClient\0done\0\0connected\0"
    "disconnected\0stateChanged\0State\0"
    "commandFinished\0error\0commandStarted\0"
    "tlsStarted\0authenticated\0rawCommandReply\0"
    "code\0details\0sslErrors\0QList<QSslError>\0"
    "socketError\0QAbstractSocket::SocketError\0"
    "err\0message\0logReceived\0data\0logSent\0"
    "disconnectFromHost\0startTls\0ignoreSslErrors\0"
    "onConnected\0onDisconnected\0onError\0"
    "_q_readFromSocket\0_q_encrypted\0"
    "Disconnected\0Connecting\0Connected\0"
    "TLSRequested\0Authenticating\0Sending\0"
    "Disconnecting\0Options\0Option\0NoOptions\0"
    "StartTlsOption\0SizeOption\0PipeliningOption\0"
    "EightBitMimeOption\0AuthOption\0AuthMode\0"
    "AuthNone\0AuthAny\0AuthPlain\0AuthLogin\0"
    "AuthModes"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QwwSmtpClient[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       4,  166, // enums/sets
       0,    0, // constructors
       0,       // flags
      13,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  119,    2, 0x06 /* Public */,
       3,    0,  122,    2, 0x06 /* Public */,
       4,    0,  123,    2, 0x06 /* Public */,
       5,    1,  124,    2, 0x06 /* Public */,
       7,    2,  127,    2, 0x06 /* Public */,
       9,    1,  132,    2, 0x06 /* Public */,
      10,    0,  135,    2, 0x06 /* Public */,
      11,    0,  136,    2, 0x06 /* Public */,
      12,    2,  137,    2, 0x06 /* Public */,
      15,    1,  142,    2, 0x06 /* Public */,
      17,    2,  145,    2, 0x06 /* Public */,
      21,    1,  150,    2, 0x06 /* Public */,
      23,    1,  153,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      24,    0,  156,    2, 0x0a /* Public */,
      25,    0,  157,    2, 0x0a /* Public */,
      26,    0,  158,    2, 0x0a /* Public */,
      27,    0,  159,    2, 0x08 /* Private */,
      28,    0,  160,    2, 0x08 /* Private */,
      29,    1,  161,    2, 0x08 /* Private */,
      30,    0,  164,    2, 0x08 /* Private */,
      31,    0,  165,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    2,    8,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   13,   14,
    QMetaType::Void, 0x80000000 | 16,    2,
    QMetaType::Void, 0x80000000 | 18, QMetaType::QString,   19,   20,
    QMetaType::Void, QMetaType::QByteArray,   22,
    QMetaType::Void, QMetaType::QByteArray,   22,

 // slots: parameters
    QMetaType::Int,
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18,    2,
    QMetaType::Void,
    QMetaType::Void,

 // enums: name, alias, flags, count, data
       6,    6, 0x0,    7,  186,
      39,   40, 0x1,    6,  200,
      47,   47, 0x0,    4,  212,
      52,   47, 0x1,    4,  220,

 // enum data: key, value
      32, uint(QwwSmtpClient::Disconnected),
      33, uint(QwwSmtpClient::Connecting),
      34, uint(QwwSmtpClient::Connected),
      35, uint(QwwSmtpClient::TLSRequested),
      36, uint(QwwSmtpClient::Authenticating),
      37, uint(QwwSmtpClient::Sending),
      38, uint(QwwSmtpClient::Disconnecting),
      41, uint(QwwSmtpClient::NoOptions),
      42, uint(QwwSmtpClient::StartTlsOption),
      43, uint(QwwSmtpClient::SizeOption),
      44, uint(QwwSmtpClient::PipeliningOption),
      45, uint(QwwSmtpClient::EightBitMimeOption),
      46, uint(QwwSmtpClient::AuthOption),
      48, uint(QwwSmtpClient::AuthNone),
      49, uint(QwwSmtpClient::AuthAny),
      50, uint(QwwSmtpClient::AuthPlain),
      51, uint(QwwSmtpClient::AuthLogin),
      48, uint(QwwSmtpClient::AuthNone),
      49, uint(QwwSmtpClient::AuthAny),
      50, uint(QwwSmtpClient::AuthPlain),
      51, uint(QwwSmtpClient::AuthLogin),

       0        // eod
};

void QwwSmtpClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QwwSmtpClient *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->done((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->connected(); break;
        case 2: _t->disconnected(); break;
        case 3: _t->stateChanged((*reinterpret_cast< State(*)>(_a[1]))); break;
        case 4: _t->commandFinished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->commandStarted((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->tlsStarted(); break;
        case 7: _t->authenticated(); break;
        case 8: _t->rawCommandReply((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 9: _t->sslErrors((*reinterpret_cast< const QList<QSslError>(*)>(_a[1]))); break;
        case 10: _t->socketError((*reinterpret_cast< QAbstractSocket::SocketError(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 11: _t->logReceived((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 12: _t->logSent((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 13: { int _r = _t->disconnectFromHost();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 14: { int _r = _t->startTls();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->ignoreSslErrors(); break;
        case 16: _t->d->onConnected(); break;
        case 17: _t->d->onDisconnected(); break;
        case 18: _t->d->onError((*reinterpret_cast< QAbstractSocket::SocketError(*)>(_a[1]))); break;
        case 19: _t->d->_q_readFromSocket(); break;
        case 20: _t->d->_q_encrypted(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QSslError> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractSocket::SocketError >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractSocket::SocketError >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QwwSmtpClient::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::done)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::connected)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::disconnected)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)(State );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::stateChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)(int , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::commandFinished)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::commandStarted)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::tlsStarted)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::authenticated)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::rawCommandReply)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)(const QList<QSslError> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::sslErrors)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)(QAbstractSocket::SocketError , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::socketError)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)(const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::logReceived)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QwwSmtpClient::*)(const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QwwSmtpClient::logSent)) {
                *result = 12;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QwwSmtpClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QwwSmtpClient.data,
    qt_meta_data_QwwSmtpClient,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QwwSmtpClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QwwSmtpClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QwwSmtpClient.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QwwSmtpClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void QwwSmtpClient::done(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QwwSmtpClient::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QwwSmtpClient::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QwwSmtpClient::stateChanged(State _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QwwSmtpClient::commandFinished(int _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QwwSmtpClient::commandStarted(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QwwSmtpClient::tlsStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QwwSmtpClient::authenticated()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QwwSmtpClient::rawCommandReply(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QwwSmtpClient::sslErrors(const QList<QSslError> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QwwSmtpClient::socketError(QAbstractSocket::SocketError _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QwwSmtpClient::logReceived(const QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QwwSmtpClient::logSent(const QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
