// This file is autogenerated. Changes will be overwritten.
#include "KAT5FKFLKL/moc_DeletionWatcher.cpp"
#include "KAT5FKFLKL/moc_FakeSocket.cpp"
#include "KAT5FKFLKL/moc_IODeviceSocket.cpp"
#include "KAT5FKFLKL/moc_Socket.cpp"
#include "KAT5FKFLKL/moc_SocketFactory.cpp"
