/****************************************************************************
** Meta object code from reading C++ file 'MailboxModel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/MailboxModel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MailboxModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Mailbox__MailboxModel_t {
    QByteArrayData data[17];
    char stringdata0[274];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__MailboxModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__MailboxModel_t qt_meta_stringdata_Imap__Mailbox__MailboxModel = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Imap::Mailbox::MailboxModel"
QT_MOC_LITERAL(1, 28, 17), // "handleDataChanged"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 59, 7), // "topLeft"
QT_MOC_LITERAL(5, 67, 11), // "bottomRight"
QT_MOC_LITERAL(6, 79, 33), // "handleMessageCountPossiblyCha..."
QT_MOC_LITERAL(7, 113, 7), // "mailbox"
QT_MOC_LITERAL(8, 121, 25), // "handleModelAboutToBeReset"
QT_MOC_LITERAL(9, 147, 16), // "handleModelReset"
QT_MOC_LITERAL(10, 164, 26), // "handleRowsAboutToBeRemoved"
QT_MOC_LITERAL(11, 191, 6), // "parent"
QT_MOC_LITERAL(12, 198, 5), // "first"
QT_MOC_LITERAL(13, 204, 4), // "last"
QT_MOC_LITERAL(14, 209, 17), // "handleRowsRemoved"
QT_MOC_LITERAL(15, 227, 27), // "handleRowsAboutToBeInserted"
QT_MOC_LITERAL(16, 255, 18) // "handleRowsInserted"

    },
    "Imap::Mailbox::MailboxModel\0"
    "handleDataChanged\0\0QModelIndex\0topLeft\0"
    "bottomRight\0handleMessageCountPossiblyChanged\0"
    "mailbox\0handleModelAboutToBeReset\0"
    "handleModelReset\0handleRowsAboutToBeRemoved\0"
    "parent\0first\0last\0handleRowsRemoved\0"
    "handleRowsAboutToBeInserted\0"
    "handleRowsInserted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__MailboxModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   54,    2, 0x09 /* Protected */,
       6,    1,   59,    2, 0x09 /* Protected */,
       8,    0,   62,    2, 0x08 /* Private */,
       9,    0,   63,    2, 0x08 /* Private */,
      10,    3,   64,    2, 0x08 /* Private */,
      14,    3,   71,    2, 0x08 /* Private */,
      15,    3,   78,    2, 0x08 /* Private */,
      16,    3,   85,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,   11,   12,   13,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,   11,   12,   13,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,   11,   12,   13,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,   11,   12,   13,

       0        // eod
};

void Imap::Mailbox::MailboxModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MailboxModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 1: _t->handleMessageCountPossiblyChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 2: _t->handleModelAboutToBeReset(); break;
        case 3: _t->handleModelReset(); break;
        case 4: _t->handleRowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->handleRowsRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 6: _t->handleRowsAboutToBeInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 7: _t->handleRowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::MailboxModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractProxyModel::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__MailboxModel.data,
    qt_meta_data_Imap__Mailbox__MailboxModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::MailboxModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::MailboxModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__MailboxModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractProxyModel::qt_metacast(_clname);
}

int Imap::Mailbox::MailboxModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
