/****************************************************************************
** Meta object code from reading C++ file 'ImapAccess.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/ImapAccess.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ImapAccess.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__ImapAccess_t {
    QByteArrayData data[61];
    char stringdata0[884];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__ImapAccess_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__ImapAccess_t qt_meta_stringdata_Imap__ImapAccess = {
    {
QT_MOC_LITERAL(0, 0, 16), // "Imap::ImapAccess"
QT_MOC_LITERAL(1, 17, 13), // "serverChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 11), // "portChanged"
QT_MOC_LITERAL(4, 44, 15), // "usernameChanged"
QT_MOC_LITERAL(5, 60, 17), // "connMethodChanged"
QT_MOC_LITERAL(6, 78, 13), // "modelsChanged"
QT_MOC_LITERAL(7, 92, 14), // "checkSslPolicy"
QT_MOC_LITERAL(8, 107, 10), // "cacheError"
QT_MOC_LITERAL(9, 118, 7), // "message"
QT_MOC_LITERAL(10, 126, 28), // "numberRefreshIntervalChanged"
QT_MOC_LITERAL(11, 155, 13), // "alertReceived"
QT_MOC_LITERAL(12, 169, 9), // "imapError"
QT_MOC_LITERAL(13, 179, 12), // "networkError"
QT_MOC_LITERAL(14, 192, 12), // "onCacheError"
QT_MOC_LITERAL(15, 205, 10), // "slotLogged"
QT_MOC_LITERAL(16, 216, 8), // "parserId"
QT_MOC_LITERAL(17, 225, 18), // "Common::LogMessage"
QT_MOC_LITERAL(18, 244, 13), // "slotSslErrors"
QT_MOC_LITERAL(19, 258, 22), // "QList<QSslCertificate>"
QT_MOC_LITERAL(20, 281, 19), // "sslCertificateChain"
QT_MOC_LITERAL(21, 301, 16), // "QList<QSslError>"
QT_MOC_LITERAL(22, 318, 9), // "sslErrors"
QT_MOC_LITERAL(23, 328, 19), // "reloadConfiguration"
QT_MOC_LITERAL(24, 348, 25), // "onRequireStartTlsInFuture"
QT_MOC_LITERAL(25, 374, 27), // "desiredNetworkPolicyChanged"
QT_MOC_LITERAL(26, 402, 28), // "Imap::Mailbox::NetworkPolicy"
QT_MOC_LITERAL(27, 431, 6), // "policy"
QT_MOC_LITERAL(28, 438, 9), // "doConnect"
QT_MOC_LITERAL(29, 448, 11), // "openMessage"
QT_MOC_LITERAL(30, 460, 7), // "mailbox"
QT_MOC_LITERAL(31, 468, 3), // "uid"
QT_MOC_LITERAL(32, 472, 12), // "setSslPolicy"
QT_MOC_LITERAL(33, 485, 6), // "accept"
QT_MOC_LITERAL(34, 492, 20), // "forgetSslCertificate"
QT_MOC_LITERAL(35, 513, 9), // "nukeCache"
QT_MOC_LITERAL(36, 523, 27), // "mailboxListShortMailboxName"
QT_MOC_LITERAL(37, 551, 22), // "mailboxListMailboxName"
QT_MOC_LITERAL(38, 574, 16), // "deproxifiedIndex"
QT_MOC_LITERAL(39, 591, 11), // "QModelIndex"
QT_MOC_LITERAL(40, 603, 5), // "index"
QT_MOC_LITERAL(41, 609, 18), // "markMessageDeleted"
QT_MOC_LITERAL(42, 628, 6), // "marked"
QT_MOC_LITERAL(43, 635, 12), // "isConfigured"
QT_MOC_LITERAL(44, 648, 9), // "imapModel"
QT_MOC_LITERAL(45, 658, 12), // "mailboxModel"
QT_MOC_LITERAL(46, 671, 12), // "msgListModel"
QT_MOC_LITERAL(47, 684, 21), // "threadingMsgListModel"
QT_MOC_LITERAL(48, 706, 17), // "visibleTasksModel"
QT_MOC_LITERAL(49, 724, 15), // "oneMessageModel"
QT_MOC_LITERAL(50, 740, 14), // "networkWatcher"
QT_MOC_LITERAL(51, 755, 7), // "msgQNAM"
QT_MOC_LITERAL(52, 763, 15), // "passwordWatcher"
QT_MOC_LITERAL(53, 779, 25), // "UiUtils::PasswordWatcher*"
QT_MOC_LITERAL(54, 805, 6), // "server"
QT_MOC_LITERAL(55, 812, 4), // "port"
QT_MOC_LITERAL(56, 817, 8), // "username"
QT_MOC_LITERAL(57, 826, 7), // "sslMode"
QT_MOC_LITERAL(58, 834, 12), // "sslInfoTitle"
QT_MOC_LITERAL(59, 847, 14), // "sslInfoMessage"
QT_MOC_LITERAL(60, 862, 21) // "numberRefreshInterval"

    },
    "Imap::ImapAccess\0serverChanged\0\0"
    "portChanged\0usernameChanged\0"
    "connMethodChanged\0modelsChanged\0"
    "checkSslPolicy\0cacheError\0message\0"
    "numberRefreshIntervalChanged\0alertReceived\0"
    "imapError\0networkError\0onCacheError\0"
    "slotLogged\0parserId\0Common::LogMessage\0"
    "slotSslErrors\0QList<QSslCertificate>\0"
    "sslCertificateChain\0QList<QSslError>\0"
    "sslErrors\0reloadConfiguration\0"
    "onRequireStartTlsInFuture\0"
    "desiredNetworkPolicyChanged\0"
    "Imap::Mailbox::NetworkPolicy\0policy\0"
    "doConnect\0openMessage\0mailbox\0uid\0"
    "setSslPolicy\0accept\0forgetSslCertificate\0"
    "nukeCache\0mailboxListShortMailboxName\0"
    "mailboxListMailboxName\0deproxifiedIndex\0"
    "QModelIndex\0index\0markMessageDeleted\0"
    "marked\0isConfigured\0imapModel\0"
    "mailboxModel\0msgListModel\0"
    "threadingMsgListModel\0visibleTasksModel\0"
    "oneMessageModel\0networkWatcher\0msgQNAM\0"
    "passwordWatcher\0UiUtils::PasswordWatcher*\0"
    "server\0port\0username\0sslMode\0sslInfoTitle\0"
    "sslInfoMessage\0numberRefreshInterval"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__ImapAccess[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
      17,  202, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  144,    2, 0x06 /* Public */,
       3,    0,  145,    2, 0x06 /* Public */,
       4,    0,  146,    2, 0x06 /* Public */,
       5,    0,  147,    2, 0x06 /* Public */,
       6,    0,  148,    2, 0x06 /* Public */,
       7,    0,  149,    2, 0x06 /* Public */,
       8,    1,  150,    2, 0x06 /* Public */,
      10,    0,  153,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    1,  154,    2, 0x0a /* Public */,
      12,    1,  157,    2, 0x0a /* Public */,
      13,    1,  160,    2, 0x0a /* Public */,
      14,    1,  163,    2, 0x0a /* Public */,
      15,    2,  166,    2, 0x0a /* Public */,
      18,    2,  171,    2, 0x0a /* Public */,
      23,    0,  176,    2, 0x0a /* Public */,
      24,    0,  177,    2, 0x08 /* Private */,
      25,    1,  178,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      28,    0,  181,    2, 0x02 /* Public */,
      29,    2,  182,    2, 0x02 /* Public */,
      32,    1,  187,    2, 0x02 /* Public */,
      34,    0,  190,    2, 0x02 /* Public */,
      35,    0,  191,    2, 0x02 /* Public */,
      36,    0,  192,    2, 0x02 /* Public */,
      37,    0,  193,    2, 0x02 /* Public */,
      38,    1,  194,    2, 0x02 /* Public */,
      41,    2,  197,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 17,   16,    9,
    QMetaType::Void, 0x80000000 | 19, 0x80000000 | 21,   20,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 26,   27,

 // methods: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::UInt,   30,   31,
    QMetaType::Void, QMetaType::Bool,   33,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QString,
    QMetaType::QString,
    0x80000000 | 39, 0x80000000 | 39,   40,
    QMetaType::Void, 0x80000000 | 39, QMetaType::Bool,    9,   42,

 // properties: name, type, flags
      43, QMetaType::Bool, 0x00495001,
      44, QMetaType::QObjectStar, 0x00495001,
      45, QMetaType::QObjectStar, 0x00495001,
      46, QMetaType::QObjectStar, 0x00495001,
      47, QMetaType::QObjectStar, 0x00495001,
      48, QMetaType::QObjectStar, 0x00495001,
      49, QMetaType::QObjectStar, 0x00495001,
      50, QMetaType::QObjectStar, 0x00495001,
      51, QMetaType::QObjectStar, 0x00495001,
      52, 0x80000000 | 53, 0x00495009,
      54, QMetaType::QString, 0x00495103,
      55, QMetaType::Int, 0x00495103,
      56, QMetaType::QString, 0x00495103,
      57, QMetaType::QString, 0x00495103,
      58, QMetaType::QString, 0x00495001,
      59, QMetaType::QString, 0x00495001,
      60, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       4,
       4,
       4,
       4,
       4,
       4,
       4,
       4,
       4,
       4,
       0,
       1,
       2,
       3,
       5,
       5,
       7,

 // enums: name, alias, flags, count, data

 // enum data: key, value

       0        // eod
};

void Imap::ImapAccess::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ImapAccess *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->serverChanged(); break;
        case 1: _t->portChanged(); break;
        case 2: _t->usernameChanged(); break;
        case 3: _t->connMethodChanged(); break;
        case 4: _t->modelsChanged(); break;
        case 5: _t->checkSslPolicy(); break;
        case 6: _t->cacheError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->numberRefreshIntervalChanged(); break;
        case 8: _t->alertReceived((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->imapError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->networkError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->onCacheError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->slotLogged((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< const Common::LogMessage(*)>(_a[2]))); break;
        case 13: _t->slotSslErrors((*reinterpret_cast< const QList<QSslCertificate>(*)>(_a[1])),(*reinterpret_cast< const QList<QSslError>(*)>(_a[2]))); break;
        case 14: _t->reloadConfiguration(); break;
        case 15: _t->onRequireStartTlsInFuture(); break;
        case 16: _t->desiredNetworkPolicyChanged((*reinterpret_cast< const Imap::Mailbox::NetworkPolicy(*)>(_a[1]))); break;
        case 17: _t->doConnect(); break;
        case 18: _t->openMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const uint(*)>(_a[2]))); break;
        case 19: _t->setSslPolicy((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->forgetSslCertificate(); break;
        case 21: _t->nukeCache(); break;
        case 22: { QString _r = _t->mailboxListShortMailboxName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 23: { QString _r = _t->mailboxListMailboxName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 24: { QModelIndex _r = _t->deproxifiedIndex((*reinterpret_cast< const QModelIndex(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->markMessageDeleted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QSslCertificate> >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QSslError> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ImapAccess::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapAccess::serverChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ImapAccess::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapAccess::portChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ImapAccess::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapAccess::usernameChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ImapAccess::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapAccess::connMethodChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ImapAccess::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapAccess::modelsChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ImapAccess::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapAccess::checkSslPolicy)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ImapAccess::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapAccess::cacheError)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (ImapAccess::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ImapAccess::numberRefreshIntervalChanged)) {
                *result = 7;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< UiUtils::PasswordWatcher* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<ImapAccess *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isConfigured(); break;
        case 1: *reinterpret_cast< QObject**>(_v) = _t->imapModel(); break;
        case 2: *reinterpret_cast< QObject**>(_v) = _t->mailboxModel(); break;
        case 3: *reinterpret_cast< QObject**>(_v) = _t->msgListModel(); break;
        case 4: *reinterpret_cast< QObject**>(_v) = _t->threadingMsgListModel(); break;
        case 5: *reinterpret_cast< QObject**>(_v) = _t->visibleTasksModel(); break;
        case 6: *reinterpret_cast< QObject**>(_v) = _t->oneMessageModel(); break;
        case 7: *reinterpret_cast< QObject**>(_v) = _t->networkWatcher(); break;
        case 8: *reinterpret_cast< QObject**>(_v) = _t->msgQNAM(); break;
        case 9: *reinterpret_cast< UiUtils::PasswordWatcher**>(_v) = _t->passwordWatcher(); break;
        case 10: *reinterpret_cast< QString*>(_v) = _t->server(); break;
        case 11: *reinterpret_cast< int*>(_v) = _t->port(); break;
        case 12: *reinterpret_cast< QString*>(_v) = _t->username(); break;
        case 13: *reinterpret_cast< QString*>(_v) = _t->sslMode(); break;
        case 14: *reinterpret_cast< QString*>(_v) = _t->sslInfoTitle(); break;
        case 15: *reinterpret_cast< QString*>(_v) = _t->sslInfoMessage(); break;
        case 16: *reinterpret_cast< int*>(_v) = _t->numberRefreshInterval(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<ImapAccess *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 10: _t->setServer(*reinterpret_cast< QString*>(_v)); break;
        case 11: _t->setPort(*reinterpret_cast< int*>(_v)); break;
        case 12: _t->setUsername(*reinterpret_cast< QString*>(_v)); break;
        case 13: _t->setSslMode(*reinterpret_cast< QString*>(_v)); break;
        case 16: _t->setNumberRefreshInterval(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Imap::ImapAccess::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Imap__ImapAccess.data,
    qt_meta_data_Imap__ImapAccess,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::ImapAccess::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::ImapAccess::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__ImapAccess.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Imap::ImapAccess::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 17;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Imap::ImapAccess::serverChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Imap::ImapAccess::portChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Imap::ImapAccess::usernameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Imap::ImapAccess::connMethodChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Imap::ImapAccess::modelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Imap::ImapAccess::checkSslPolicy()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Imap::ImapAccess::cacheError(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Imap::ImapAccess::numberRefreshIntervalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
