/********************************************************************************
** Form generated from reading UI file 'be-contacts.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BE_2D_CONTACTS_H
#define UI_BE_2D_CONTACTS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Contacts
{
public:
    QGridLayout *gridLayout;
    QSplitter *splitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QLineEdit *filter;
    QListView *contacts;
    QHBoxLayout *horizontalLayout;
    QPushButton *remove;
    QPushButton *add;
    QWidget *oneContact;

    void setupUi(QWidget *Contacts)
    {
        if (Contacts->objectName().isEmpty())
            Contacts->setObjectName(QString::fromUtf8("Contacts"));
        Contacts->resize(266, 259);
        gridLayout = new QGridLayout(Contacts);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        splitter = new QSplitter(Contacts);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Horizontal);
        layoutWidget = new QWidget(splitter);
        layoutWidget->setObjectName(QString::fromUtf8("layoutWidget"));
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        filter = new QLineEdit(layoutWidget);
        filter->setObjectName(QString::fromUtf8("filter"));

        verticalLayout->addWidget(filter);

        contacts = new QListView(layoutWidget);
        contacts->setObjectName(QString::fromUtf8("contacts"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(contacts->sizePolicy().hasHeightForWidth());
        contacts->setSizePolicy(sizePolicy);
        contacts->setMinimumSize(QSize(200, 0));

        verticalLayout->addWidget(contacts);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        remove = new QPushButton(layoutWidget);
        remove->setObjectName(QString::fromUtf8("remove"));
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("edit-delete")));
        remove->setIcon(icon);

        horizontalLayout->addWidget(remove);

        add = new QPushButton(layoutWidget);
        add->setObjectName(QString::fromUtf8("add"));

        horizontalLayout->addWidget(add);


        verticalLayout->addLayout(horizontalLayout);

        splitter->addWidget(layoutWidget);
        oneContact = new QWidget(splitter);
        oneContact->setObjectName(QString::fromUtf8("oneContact"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(oneContact->sizePolicy().hasHeightForWidth());
        oneContact->setSizePolicy(sizePolicy1);
        splitter->addWidget(oneContact);

        gridLayout->addWidget(splitter, 0, 0, 1, 1);

        QWidget::setTabOrder(filter, contacts);
        QWidget::setTabOrder(contacts, remove);
        QWidget::setTabOrder(remove, add);

        retranslateUi(Contacts);

        add->setDefault(true);


        QMetaObject::connectSlotsByName(Contacts);
    } // setupUi

    void retranslateUi(QWidget *Contacts)
    {
        Contacts->setWindowTitle(QCoreApplication::translate("Contacts", "Contacts", nullptr));
        remove->setText(QCoreApplication::translate("Contacts", "Delete", nullptr));
        add->setText(QCoreApplication::translate("Contacts", "Add", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Contacts: public Ui_Contacts {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BE_2D_CONTACTS_H
