/****************************************************************************
** Meta object code from reading C++ file 'SocketFactory.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Streams/SocketFactory.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SocketFactory.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Streams__SocketFactory_t {
    QByteArrayData data[3];
    char stringdata0[30];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Streams__SocketFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Streams__SocketFactory_t qt_meta_stringdata_Streams__SocketFactory = {
    {
QT_MOC_LITERAL(0, 0, 22), // "Streams::SocketFactory"
QT_MOC_LITERAL(1, 23, 5), // "error"
QT_MOC_LITERAL(2, 29, 0) // ""

    },
    "Streams::SocketFactory\0error\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Streams__SocketFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void Streams::SocketFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SocketFactory *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SocketFactory::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SocketFactory::error)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Streams::SocketFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Streams__SocketFactory.data,
    qt_meta_data_Streams__SocketFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Streams::SocketFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Streams::SocketFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Streams__SocketFactory.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Streams::SocketFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Streams::SocketFactory::error(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_Streams__ProcessSocketFactory_t {
    QByteArrayData data[1];
    char stringdata0[30];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Streams__ProcessSocketFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Streams__ProcessSocketFactory_t qt_meta_stringdata_Streams__ProcessSocketFactory = {
    {
QT_MOC_LITERAL(0, 0, 29) // "Streams::ProcessSocketFactory"

    },
    "Streams::ProcessSocketFactory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Streams__ProcessSocketFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Streams::ProcessSocketFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Streams::ProcessSocketFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<SocketFactory::staticMetaObject>(),
    qt_meta_stringdata_Streams__ProcessSocketFactory.data,
    qt_meta_data_Streams__ProcessSocketFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Streams::ProcessSocketFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Streams::ProcessSocketFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Streams__ProcessSocketFactory.stringdata0))
        return static_cast<void*>(this);
    return SocketFactory::qt_metacast(_clname);
}

int Streams::ProcessSocketFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SocketFactory::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Streams__SslSocketFactory_t {
    QByteArrayData data[1];
    char stringdata0[26];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Streams__SslSocketFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Streams__SslSocketFactory_t qt_meta_stringdata_Streams__SslSocketFactory = {
    {
QT_MOC_LITERAL(0, 0, 25) // "Streams::SslSocketFactory"

    },
    "Streams::SslSocketFactory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Streams__SslSocketFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Streams::SslSocketFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Streams::SslSocketFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<SocketFactory::staticMetaObject>(),
    qt_meta_stringdata_Streams__SslSocketFactory.data,
    qt_meta_data_Streams__SslSocketFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Streams::SslSocketFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Streams::SslSocketFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Streams__SslSocketFactory.stringdata0))
        return static_cast<void*>(this);
    return SocketFactory::qt_metacast(_clname);
}

int Streams::SslSocketFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SocketFactory::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Streams__TlsAbleSocketFactory_t {
    QByteArrayData data[1];
    char stringdata0[30];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Streams__TlsAbleSocketFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Streams__TlsAbleSocketFactory_t qt_meta_stringdata_Streams__TlsAbleSocketFactory = {
    {
QT_MOC_LITERAL(0, 0, 29) // "Streams::TlsAbleSocketFactory"

    },
    "Streams::TlsAbleSocketFactory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Streams__TlsAbleSocketFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Streams::TlsAbleSocketFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Streams::TlsAbleSocketFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<SocketFactory::staticMetaObject>(),
    qt_meta_stringdata_Streams__TlsAbleSocketFactory.data,
    qt_meta_data_Streams__TlsAbleSocketFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Streams::TlsAbleSocketFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Streams::TlsAbleSocketFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Streams__TlsAbleSocketFactory.stringdata0))
        return static_cast<void*>(this);
    return SocketFactory::qt_metacast(_clname);
}

int Streams::TlsAbleSocketFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SocketFactory::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Streams__FakeSocketFactory_t {
    QByteArrayData data[1];
    char stringdata0[27];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Streams__FakeSocketFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Streams__FakeSocketFactory_t qt_meta_stringdata_Streams__FakeSocketFactory = {
    {
QT_MOC_LITERAL(0, 0, 26) // "Streams::FakeSocketFactory"

    },
    "Streams::FakeSocketFactory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Streams__FakeSocketFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Streams::FakeSocketFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Streams::FakeSocketFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<SocketFactory::staticMetaObject>(),
    qt_meta_stringdata_Streams__FakeSocketFactory.data,
    qt_meta_data_Streams__FakeSocketFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Streams::FakeSocketFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Streams::FakeSocketFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Streams__FakeSocketFactory.stringdata0))
        return static_cast<void*>(this);
    return SocketFactory::qt_metacast(_clname);
}

int Streams::FakeSocketFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SocketFactory::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
