/********************************************************************************
** Form generated from reading UI file 'EditIdentity.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITIDENTITY_H
#define UI_EDITIDENTITY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QWidget>
#include "LineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_EditIdentity
{
public:
    QFormLayout *formLayout;
    QLabel *realNameLabel;
    LineEdit *realNameLineEdit;
    QLabel *emailLabel;
    LineEdit *emailLineEdit;
    LineEdit *organisationLineEdit;
    QLabel *organisationLabel;
    QLabel *signatureLabel;
    QPlainTextEdit *signaturePlainTextEdit;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *EditIdentity)
    {
        if (EditIdentity->objectName().isEmpty())
            EditIdentity->setObjectName(QString::fromUtf8("EditIdentity"));
        EditIdentity->resize(700, 309);
        EditIdentity->setMinimumSize(QSize(700, 0));
        formLayout = new QFormLayout(EditIdentity);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        realNameLabel = new QLabel(EditIdentity);
        realNameLabel->setObjectName(QString::fromUtf8("realNameLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, realNameLabel);

        realNameLineEdit = new LineEdit(EditIdentity);
        realNameLineEdit->setObjectName(QString::fromUtf8("realNameLineEdit"));

        formLayout->setWidget(0, QFormLayout::FieldRole, realNameLineEdit);

        emailLabel = new QLabel(EditIdentity);
        emailLabel->setObjectName(QString::fromUtf8("emailLabel"));

        formLayout->setWidget(1, QFormLayout::LabelRole, emailLabel);

        emailLineEdit = new LineEdit(EditIdentity);
        emailLineEdit->setObjectName(QString::fromUtf8("emailLineEdit"));

        formLayout->setWidget(1, QFormLayout::FieldRole, emailLineEdit);

        organisationLineEdit = new LineEdit(EditIdentity);
        organisationLineEdit->setObjectName(QString::fromUtf8("organisationLineEdit"));

        formLayout->setWidget(2, QFormLayout::FieldRole, organisationLineEdit);

        organisationLabel = new QLabel(EditIdentity);
        organisationLabel->setObjectName(QString::fromUtf8("organisationLabel"));

        formLayout->setWidget(2, QFormLayout::LabelRole, organisationLabel);

        signatureLabel = new QLabel(EditIdentity);
        signatureLabel->setObjectName(QString::fromUtf8("signatureLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, signatureLabel);

        signaturePlainTextEdit = new QPlainTextEdit(EditIdentity);
        signaturePlainTextEdit->setObjectName(QString::fromUtf8("signaturePlainTextEdit"));
        signaturePlainTextEdit->setLineWrapMode(QPlainTextEdit::NoWrap);

        formLayout->setWidget(3, QFormLayout::FieldRole, signaturePlainTextEdit);

        buttonBox = new QDialogButtonBox(EditIdentity);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        formLayout->setWidget(4, QFormLayout::FieldRole, buttonBox);


        retranslateUi(EditIdentity);

        QMetaObject::connectSlotsByName(EditIdentity);
    } // setupUi

    void retranslateUi(QWidget *EditIdentity)
    {
        EditIdentity->setWindowTitle(QCoreApplication::translate("EditIdentity", "Manage Identity", nullptr));
        realNameLabel->setText(QCoreApplication::translate("EditIdentity", "Real Name", nullptr));
        emailLabel->setText(QCoreApplication::translate("EditIdentity", "E-mail", nullptr));
        organisationLabel->setText(QCoreApplication::translate("EditIdentity", "Organisation", nullptr));
        signatureLabel->setText(QCoreApplication::translate("EditIdentity", "Signature", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditIdentity: public Ui_EditIdentity {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITIDENTITY_H
