/********************************************************************************
** Form generated from reading UI file 'ComposeWidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COMPOSEWIDGET_H
#define UI_COMPOSEWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollBar>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ComposerAttachmentsList.h"
#include "ComposerTextEdit.h"
#include "LineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_ComposeWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QSplitter *verticalSplitter;
    QSplitter *splitter;
    QWidget *envelopeWidget;
    QHBoxLayout *horizontalLayout;
    QFormLayout *envelopeLayout;
    QLabel *fromLabel;
    QComboBox *sender;
    QLabel *subjectLabel;
    LineEdit *subject;
    QScrollBar *recipientSlider;
    QGroupBox *attachmentBox;
    QVBoxLayout *verticalLayout;
    QPushButton *attachButton;
    ComposerAttachmentsList *attachmentsView;
    ComposerTextEdit *mailText;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *ComposeWidget)
    {
        if (ComposeWidget->objectName().isEmpty())
            ComposeWidget->setObjectName(QString::fromUtf8("ComposeWidget"));
        ComposeWidget->resize(714, 361);
        verticalLayout_2 = new QVBoxLayout(ComposeWidget);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalSplitter = new QSplitter(ComposeWidget);
        verticalSplitter->setObjectName(QString::fromUtf8("verticalSplitter"));
        verticalSplitter->setOrientation(Qt::Vertical);
        verticalSplitter->setChildrenCollapsible(false);
        splitter = new QSplitter(verticalSplitter);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(splitter->sizePolicy().hasHeightForWidth());
        splitter->setSizePolicy(sizePolicy);
        splitter->setOrientation(Qt::Horizontal);
        splitter->setChildrenCollapsible(false);
        envelopeWidget = new QWidget(splitter);
        envelopeWidget->setObjectName(QString::fromUtf8("envelopeWidget"));
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(4);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(envelopeWidget->sizePolicy().hasHeightForWidth());
        envelopeWidget->setSizePolicy(sizePolicy1);
        horizontalLayout = new QHBoxLayout(envelopeWidget);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        envelopeLayout = new QFormLayout();
        envelopeLayout->setObjectName(QString::fromUtf8("envelopeLayout"));
        envelopeLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        fromLabel = new QLabel(envelopeWidget);
        fromLabel->setObjectName(QString::fromUtf8("fromLabel"));

        envelopeLayout->setWidget(0, QFormLayout::LabelRole, fromLabel);

        sender = new QComboBox(envelopeWidget);
        sender->setObjectName(QString::fromUtf8("sender"));
        QSizePolicy sizePolicy2(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(sender->sizePolicy().hasHeightForWidth());
        sender->setSizePolicy(sizePolicy2);
        sender->setEditable(true);
        sender->setInsertPolicy(QComboBox::NoInsert);

        envelopeLayout->setWidget(0, QFormLayout::FieldRole, sender);

        subjectLabel = new QLabel(envelopeWidget);
        subjectLabel->setObjectName(QString::fromUtf8("subjectLabel"));

        envelopeLayout->setWidget(1, QFormLayout::LabelRole, subjectLabel);

        subject = new LineEdit(envelopeWidget);
        subject->setObjectName(QString::fromUtf8("subject"));

        envelopeLayout->setWidget(1, QFormLayout::FieldRole, subject);


        horizontalLayout->addLayout(envelopeLayout);

        recipientSlider = new QScrollBar(envelopeWidget);
        recipientSlider->setObjectName(QString::fromUtf8("recipientSlider"));
        recipientSlider->setOrientation(Qt::Vertical);

        horizontalLayout->addWidget(recipientSlider);

        splitter->addWidget(envelopeWidget);
        attachmentBox = new QGroupBox(splitter);
        attachmentBox->setObjectName(QString::fromUtf8("attachmentBox"));
        QSizePolicy sizePolicy3(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(1);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(attachmentBox->sizePolicy().hasHeightForWidth());
        attachmentBox->setSizePolicy(sizePolicy3);
        attachmentBox->setFlat(false);
        verticalLayout = new QVBoxLayout(attachmentBox);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        attachButton = new QPushButton(attachmentBox);
        attachButton->setObjectName(QString::fromUtf8("attachButton"));
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("mail-attachment")));
        attachButton->setIcon(icon);

        verticalLayout->addWidget(attachButton);

        attachmentsView = new ComposerAttachmentsList(attachmentBox);
        attachmentsView->setObjectName(QString::fromUtf8("attachmentsView"));
        QSizePolicy sizePolicy4(QSizePolicy::Expanding, QSizePolicy::MinimumExpanding);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(attachmentsView->sizePolicy().hasHeightForWidth());
        attachmentsView->setSizePolicy(sizePolicy4);
        attachmentsView->setMinimumSize(QSize(100, 33));
        attachmentsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        attachmentsView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        attachmentsView->setDragEnabled(true);
        attachmentsView->setDragDropMode(QAbstractItemView::DragDrop);
        attachmentsView->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
        attachmentsView->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
        attachmentsView->setFlow(QListView::TopToBottom);
        attachmentsView->setProperty("isWrapping", QVariant(false));
        attachmentsView->setResizeMode(QListView::Adjust);
        attachmentsView->setWordWrap(true);

        verticalLayout->addWidget(attachmentsView);

        verticalLayout->setStretch(0, 1);
        verticalLayout->setStretch(1, 100);
        splitter->addWidget(attachmentBox);
        verticalSplitter->addWidget(splitter);
        mailText = new ComposerTextEdit(verticalSplitter);
        mailText->setObjectName(QString::fromUtf8("mailText"));
        QSizePolicy sizePolicy5(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(100);
        sizePolicy5.setHeightForWidth(mailText->sizePolicy().hasHeightForWidth());
        mailText->setSizePolicy(sizePolicy5);
        verticalSplitter->addWidget(mailText);

        verticalLayout_2->addWidget(verticalSplitter);

        buttonBox = new QDialogButtonBox(ComposeWidget);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::NoButton);

        verticalLayout_2->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        fromLabel->setBuddy(sender);
        subjectLabel->setBuddy(subject);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(sender, subject);
        QWidget::setTabOrder(subject, mailText);

        retranslateUi(ComposeWidget);

        QMetaObject::connectSlotsByName(ComposeWidget);
    } // setupUi

    void retranslateUi(QWidget *ComposeWidget)
    {
        fromLabel->setText(QCoreApplication::translate("ComposeWidget", "From", nullptr));
        subjectLabel->setText(QCoreApplication::translate("ComposeWidget", "Subject", nullptr));
        attachmentBox->setTitle(QString());
        attachButton->setText(QCoreApplication::translate("ComposeWidget", "Attach...", nullptr));
        (void)ComposeWidget;
    } // retranslateUi

};

namespace Ui {
    class ComposeWidget: public Ui_ComposeWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_COMPOSEWIDGET_H
