/****************************************************************************
** Meta object code from reading C++ file 'NetworkWatcher.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/NetworkWatcher.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NetworkWatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Mailbox__NetworkWatcher_t {
    QByteArrayData data[18];
    char stringdata0[330];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__NetworkWatcher_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__NetworkWatcher_t qt_meta_stringdata_Imap__Mailbox__NetworkWatcher = {
    {
QT_MOC_LITERAL(0, 0, 29), // "Imap::Mailbox::NetworkWatcher"
QT_MOC_LITERAL(1, 30, 29), // "effectiveNetworkPolicyChanged"
QT_MOC_LITERAL(2, 60, 0), // ""
QT_MOC_LITERAL(3, 61, 25), // "reconnectAttemptScheduled"
QT_MOC_LITERAL(4, 87, 7), // "timeout"
QT_MOC_LITERAL(5, 95, 19), // "resetReconnectState"
QT_MOC_LITERAL(6, 115, 27), // "desiredNetworkPolicyChanged"
QT_MOC_LITERAL(7, 143, 28), // "Imap::Mailbox::NetworkPolicy"
QT_MOC_LITERAL(8, 172, 6), // "policy"
QT_MOC_LITERAL(9, 179, 17), // "setNetworkOffline"
QT_MOC_LITERAL(10, 197, 19), // "setNetworkExpensive"
QT_MOC_LITERAL(11, 217, 16), // "setNetworkOnline"
QT_MOC_LITERAL(12, 234, 16), // "attemptReconnect"
QT_MOC_LITERAL(13, 251, 12), // "tryReconnect"
QT_MOC_LITERAL(14, 264, 28), // "handleConnectionStateChanged"
QT_MOC_LITERAL(15, 293, 8), // "parserId"
QT_MOC_LITERAL(16, 302, 21), // "Imap::ConnectionState"
QT_MOC_LITERAL(17, 324, 5) // "state"

    },
    "Imap::Mailbox::NetworkWatcher\0"
    "effectiveNetworkPolicyChanged\0\0"
    "reconnectAttemptScheduled\0timeout\0"
    "resetReconnectState\0desiredNetworkPolicyChanged\0"
    "Imap::Mailbox::NetworkPolicy\0policy\0"
    "setNetworkOffline\0setNetworkExpensive\0"
    "setNetworkOnline\0attemptReconnect\0"
    "tryReconnect\0handleConnectionStateChanged\0"
    "parserId\0Imap::ConnectionState\0state"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__NetworkWatcher[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,
       3,    1,   65,    2, 0x06 /* Public */,
       5,    0,   68,    2, 0x06 /* Public */,
       6,    1,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   72,    2, 0x0a /* Public */,
      10,    0,   73,    2, 0x0a /* Public */,
      11,    0,   74,    2, 0x0a /* Public */,
      12,    0,   75,    2, 0x09 /* Protected */,
      13,    0,   76,    2, 0x09 /* Protected */,
      14,    2,   77,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 16,   15,   17,

       0        // eod
};

void Imap::Mailbox::NetworkWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NetworkWatcher *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->effectiveNetworkPolicyChanged(); break;
        case 1: _t->reconnectAttemptScheduled((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 2: _t->resetReconnectState(); break;
        case 3: _t->desiredNetworkPolicyChanged((*reinterpret_cast< const Imap::Mailbox::NetworkPolicy(*)>(_a[1]))); break;
        case 4: _t->setNetworkOffline(); break;
        case 5: _t->setNetworkExpensive(); break;
        case 6: _t->setNetworkOnline(); break;
        case 7: _t->attemptReconnect(); break;
        case 8: _t->tryReconnect(); break;
        case 9: _t->handleConnectionStateChanged((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< Imap::ConnectionState(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (NetworkWatcher::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NetworkWatcher::effectiveNetworkPolicyChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (NetworkWatcher::*)(const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NetworkWatcher::reconnectAttemptScheduled)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (NetworkWatcher::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NetworkWatcher::resetReconnectState)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (NetworkWatcher::*)(const Imap::Mailbox::NetworkPolicy );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NetworkWatcher::desiredNetworkPolicyChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::NetworkWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__NetworkWatcher.data,
    qt_meta_data_Imap__Mailbox__NetworkWatcher,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::NetworkWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::NetworkWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__NetworkWatcher.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Imap::Mailbox::NetworkWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Imap::Mailbox::NetworkWatcher::effectiveNetworkPolicyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Imap::Mailbox::NetworkWatcher::reconnectAttemptScheduled(const int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Imap::Mailbox::NetworkWatcher::resetReconnectState()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Imap::Mailbox::NetworkWatcher::desiredNetworkPolicyChanged(const Imap::Mailbox::NetworkPolicy _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
