/********************************************************************************
** Form generated from reading UI file 'SettingsCachePage.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGSCACHEPAGE_H
#define UI_SETTINGSCACHEPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_CachePage
{
public:
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout;
    QGroupBox *offlineModeGroup;
    QFormLayout *formLayout;
    QRadioButton *offlineNope;
    QRadioButton *offlineXDays;
    QSpinBox *offlineNumberOfDays;
    QRadioButton *offlineEverything;
    QGroupBox *notificationsBox;
    QFormLayout *formLayout_2;
    QRadioButton *watchInbox;
    QRadioButton *watchAll;
    QRadioButton *watchSubscribed;
    QSpacerItem *verticalSpacer;

    void setupUi(QScrollArea *CachePage)
    {
        if (CachePage->objectName().isEmpty())
            CachePage->setObjectName(QString::fromUtf8("CachePage"));
        CachePage->setFrameShadow(QFrame::Plain);
        CachePage->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 230, 176));
        verticalLayout = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        offlineModeGroup = new QGroupBox(scrollAreaWidgetContents);
        offlineModeGroup->setObjectName(QString::fromUtf8("offlineModeGroup"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(offlineModeGroup->sizePolicy().hasHeightForWidth());
        offlineModeGroup->setSizePolicy(sizePolicy);
        formLayout = new QFormLayout(offlineModeGroup);
        formLayout->setContentsMargins(12, 12, 12, 12);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        offlineNope = new QRadioButton(offlineModeGroup);
        offlineNope->setObjectName(QString::fromUtf8("offlineNope"));
        sizePolicy.setHeightForWidth(offlineNope->sizePolicy().hasHeightForWidth());
        offlineNope->setSizePolicy(sizePolicy);

        formLayout->setWidget(0, QFormLayout::LabelRole, offlineNope);

        offlineXDays = new QRadioButton(offlineModeGroup);
        offlineXDays->setObjectName(QString::fromUtf8("offlineXDays"));
        sizePolicy.setHeightForWidth(offlineXDays->sizePolicy().hasHeightForWidth());
        offlineXDays->setSizePolicy(sizePolicy);

        formLayout->setWidget(1, QFormLayout::LabelRole, offlineXDays);

        offlineNumberOfDays = new QSpinBox(offlineModeGroup);
        offlineNumberOfDays->setObjectName(QString::fromUtf8("offlineNumberOfDays"));
        sizePolicy.setHeightForWidth(offlineNumberOfDays->sizePolicy().hasHeightForWidth());
        offlineNumberOfDays->setSizePolicy(sizePolicy);
        offlineNumberOfDays->setMinimum(1);
        offlineNumberOfDays->setMaximum(365000);

        formLayout->setWidget(2, QFormLayout::LabelRole, offlineNumberOfDays);

        offlineEverything = new QRadioButton(offlineModeGroup);
        offlineEverything->setObjectName(QString::fromUtf8("offlineEverything"));
        sizePolicy.setHeightForWidth(offlineEverything->sizePolicy().hasHeightForWidth());
        offlineEverything->setSizePolicy(sizePolicy);

        formLayout->setWidget(3, QFormLayout::LabelRole, offlineEverything);


        verticalLayout->addWidget(offlineModeGroup);

        notificationsBox = new QGroupBox(scrollAreaWidgetContents);
        notificationsBox->setObjectName(QString::fromUtf8("notificationsBox"));
        sizePolicy.setHeightForWidth(notificationsBox->sizePolicy().hasHeightForWidth());
        notificationsBox->setSizePolicy(sizePolicy);
        formLayout_2 = new QFormLayout(notificationsBox);
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        watchInbox = new QRadioButton(notificationsBox);
        watchInbox->setObjectName(QString::fromUtf8("watchInbox"));

        formLayout_2->setWidget(0, QFormLayout::LabelRole, watchInbox);

        watchAll = new QRadioButton(notificationsBox);
        watchAll->setObjectName(QString::fromUtf8("watchAll"));

        formLayout_2->setWidget(2, QFormLayout::LabelRole, watchAll);

        watchSubscribed = new QRadioButton(notificationsBox);
        watchSubscribed->setObjectName(QString::fromUtf8("watchSubscribed"));

        formLayout_2->setWidget(1, QFormLayout::LabelRole, watchSubscribed);


        verticalLayout->addWidget(notificationsBox);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        CachePage->setWidget(scrollAreaWidgetContents);

        retranslateUi(CachePage);

        QMetaObject::connectSlotsByName(CachePage);
    } // setupUi

    void retranslateUi(QScrollArea *CachePage)
    {
        offlineModeGroup->setTitle(QCoreApplication::translate("CachePage", "Offline mode", nullptr));
        offlineNope->setText(QCoreApplication::translate("CachePage", "&Don't use cache", nullptr));
        offlineXDays->setText(QCoreApplication::translate("CachePage", "&Remember messages for X days", nullptr));
        offlineEverything->setText(QCoreApplication::translate("CachePage", "Store &forever", nullptr));
        notificationsBox->setTitle(QCoreApplication::translate("CachePage", "Notifications", nullptr));
#if QT_CONFIG(whatsthis)
        watchInbox->setWhatsThis(QCoreApplication::translate("CachePage", "Only the INBOX folder will be included for new message checking", nullptr));
#endif // QT_CONFIG(whatsthis)
        watchInbox->setText(QCoreApplication::translate("CachePage", "&INBOX Only", nullptr));
#if QT_CONFIG(whatsthis)
        watchAll->setWhatsThis(QCoreApplication::translate("CachePage", "This might result in extremely slow operation in certain scenarios with shared mailboxes, rich folder hierarchies, or IMAP proxying, for example.", nullptr));
#endif // QT_CONFIG(whatsthis)
        watchAll->setText(QCoreApplication::translate("CachePage", "&All Mailboxes (use with care)", nullptr));
#if QT_CONFIG(whatsthis)
        watchSubscribed->setWhatsThis(QCoreApplication::translate("CachePage", "E-mail folders which are marked as subscribed will be included in notifications. To subscribe a mailbox, use the context menu in the mailbox view.", nullptr));
#endif // QT_CONFIG(whatsthis)
        watchSubscribed->setText(QCoreApplication::translate("CachePage", "&Subscribed Mailboxes", nullptr));
        (void)CachePage;
    } // retranslateUi

};

namespace Ui {
    class CachePage: public Ui_CachePage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGSCACHEPAGE_H
