/****************************************************************************
** Meta object code from reading C++ file 'ComposeWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/ComposeWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ComposeWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__ComposeWidget_t {
    QByteArrayData data[53];
    char stringdata0[757];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__ComposeWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__ComposeWidget_t qt_meta_stringdata_Gui__ComposeWidget = {
    {
QT_MOC_LITERAL(0, 0, 18), // "Gui::ComposeWidget"
QT_MOC_LITERAL(1, 19, 6), // "logged"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 15), // "Common::LogKind"
QT_MOC_LITERAL(4, 43, 4), // "kind"
QT_MOC_LITERAL(5, 48, 6), // "source"
QT_MOC_LITERAL(6, 55, 7), // "message"
QT_MOC_LITERAL(7, 63, 29), // "calculateMaxVisibleRecipients"
QT_MOC_LITERAL(8, 93, 18), // "collapseRecipients"
QT_MOC_LITERAL(9, 112, 17), // "completeRecipient"
QT_MOC_LITERAL(10, 130, 8), // "QAction*"
QT_MOC_LITERAL(11, 139, 3), // "act"
QT_MOC_LITERAL(12, 143, 18), // "completeRecipients"
QT_MOC_LITERAL(13, 162, 4), // "text"
QT_MOC_LITERAL(14, 167, 4), // "send"
QT_MOC_LITERAL(15, 172, 8), // "gotError"
QT_MOC_LITERAL(16, 181, 5), // "error"
QT_MOC_LITERAL(17, 187, 4), // "sent"
QT_MOC_LITERAL(18, 192, 19), // "updateRecipientList"
QT_MOC_LITERAL(19, 212, 16), // "scrollRecipients"
QT_MOC_LITERAL(20, 229, 21), // "gotoNextInputLineFrom"
QT_MOC_LITERAL(21, 251, 8), // "QWidget*"
QT_MOC_LITERAL(22, 260, 1), // "w"
QT_MOC_LITERAL(23, 262, 17), // "handleFocusChange"
QT_MOC_LITERAL(24, 280, 13), // "scrollToFocus"
QT_MOC_LITERAL(25, 294, 16), // "slotFadeFinished"
QT_MOC_LITERAL(26, 311, 24), // "slotCheckAddressOfSender"
QT_MOC_LITERAL(27, 336, 16), // "slotCheckAddress"
QT_MOC_LITERAL(28, 353, 10), // "QLineEdit*"
QT_MOC_LITERAL(29, 364, 4), // "edit"
QT_MOC_LITERAL(30, 369, 24), // "slotAskForFileAttachment"
QT_MOC_LITERAL(31, 394, 15), // "slotAttachFiles"
QT_MOC_LITERAL(32, 410, 11), // "QList<QUrl>"
QT_MOC_LITERAL(33, 422, 4), // "urls"
QT_MOC_LITERAL(34, 427, 19), // "slotUpdateSignature"
QT_MOC_LITERAL(35, 447, 17), // "updateWindowTitle"
QT_MOC_LITERAL(36, 465, 13), // "autoSaveDraft"
QT_MOC_LITERAL(37, 479, 17), // "setMessageUpdated"
QT_MOC_LITERAL(38, 497, 19), // "setUiWidgetsEnabled"
QT_MOC_LITERAL(39, 517, 7), // "enabled"
QT_MOC_LITERAL(40, 525, 21), // "onCompletionAvailable"
QT_MOC_LITERAL(41, 547, 22), // "Plugins::NameEmailList"
QT_MOC_LITERAL(42, 570, 10), // "completion"
QT_MOC_LITERAL(43, 581, 18), // "onCompletionFailed"
QT_MOC_LITERAL(44, 600, 30), // "Plugins::AddressbookJob::Error"
QT_MOC_LITERAL(45, 631, 17), // "passwordRequested"
QT_MOC_LITERAL(46, 649, 4), // "user"
QT_MOC_LITERAL(47, 654, 4), // "host"
QT_MOC_LITERAL(48, 659, 13), // "passwordError"
QT_MOC_LITERAL(49, 673, 18), // "toggleReplyMarking"
QT_MOC_LITERAL(50, 692, 24), // "updateReplyMarkingAction"
QT_MOC_LITERAL(51, 717, 15), // "updateReplyMode"
QT_MOC_LITERAL(52, 733, 23) // "markReplyModeHandpicked"

    },
    "Gui::ComposeWidget\0logged\0\0Common::LogKind\0"
    "kind\0source\0message\0calculateMaxVisibleRecipients\0"
    "collapseRecipients\0completeRecipient\0"
    "QAction*\0act\0completeRecipients\0text\0"
    "send\0gotError\0error\0sent\0updateRecipientList\0"
    "scrollRecipients\0gotoNextInputLineFrom\0"
    "QWidget*\0w\0handleFocusChange\0scrollToFocus\0"
    "slotFadeFinished\0slotCheckAddressOfSender\0"
    "slotCheckAddress\0QLineEdit*\0edit\0"
    "slotAskForFileAttachment\0slotAttachFiles\0"
    "QList<QUrl>\0urls\0slotUpdateSignature\0"
    "updateWindowTitle\0autoSaveDraft\0"
    "setMessageUpdated\0setUiWidgetsEnabled\0"
    "enabled\0onCompletionAvailable\0"
    "Plugins::NameEmailList\0completion\0"
    "onCompletionFailed\0Plugins::AddressbookJob::Error\0"
    "passwordRequested\0user\0host\0passwordError\0"
    "toggleReplyMarking\0updateReplyMarkingAction\0"
    "updateReplyMode\0markReplyModeHandpicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__ComposeWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      31,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,  169,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,  176,    2, 0x08 /* Private */,
       8,    0,  177,    2, 0x08 /* Private */,
       9,    1,  178,    2, 0x08 /* Private */,
      12,    1,  181,    2, 0x08 /* Private */,
      14,    0,  184,    2, 0x08 /* Private */,
      15,    1,  185,    2, 0x08 /* Private */,
      17,    0,  188,    2, 0x08 /* Private */,
      18,    0,  189,    2, 0x08 /* Private */,
      19,    1,  190,    2, 0x08 /* Private */,
      20,    1,  193,    2, 0x08 /* Private */,
      23,    0,  196,    2, 0x08 /* Private */,
      24,    0,  197,    2, 0x08 /* Private */,
      25,    0,  198,    2, 0x08 /* Private */,
      26,    0,  199,    2, 0x08 /* Private */,
      27,    1,  200,    2, 0x08 /* Private */,
      30,    0,  203,    2, 0x08 /* Private */,
      31,    1,  204,    2, 0x08 /* Private */,
      34,    0,  207,    2, 0x08 /* Private */,
      35,    0,  208,    2, 0x08 /* Private */,
      36,    0,  209,    2, 0x08 /* Private */,
      37,    0,  210,    2, 0x08 /* Private */,
      38,    1,  211,    2, 0x08 /* Private */,
      40,    1,  214,    2, 0x08 /* Private */,
      43,    1,  217,    2, 0x08 /* Private */,
      45,    2,  220,    2, 0x08 /* Private */,
      48,    0,  225,    2, 0x08 /* Private */,
      49,    0,  226,    2, 0x08 /* Private */,
      50,    0,  227,    2, 0x08 /* Private */,
      51,    0,  228,    2, 0x08 /* Private */,
      52,    0,  229,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, QMetaType::QString,    4,    5,    6,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 32,   33,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   39,
    QMetaType::Void, 0x80000000 | 41,   42,
    QMetaType::Void, 0x80000000 | 44,   16,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   46,   47,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::ComposeWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ComposeWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->logged((*reinterpret_cast< const Common::LogKind(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 1: _t->calculateMaxVisibleRecipients(); break;
        case 2: _t->collapseRecipients(); break;
        case 3: _t->completeRecipient((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 4: _t->completeRecipients((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->send(); break;
        case 6: _t->gotError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->sent(); break;
        case 8: _t->updateRecipientList(); break;
        case 9: _t->scrollRecipients((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->gotoNextInputLineFrom((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 11: _t->handleFocusChange(); break;
        case 12: _t->scrollToFocus(); break;
        case 13: _t->slotFadeFinished(); break;
        case 14: _t->slotCheckAddressOfSender(); break;
        case 15: _t->slotCheckAddress((*reinterpret_cast< QLineEdit*(*)>(_a[1]))); break;
        case 16: _t->slotAskForFileAttachment(); break;
        case 17: _t->slotAttachFiles((*reinterpret_cast< QList<QUrl>(*)>(_a[1]))); break;
        case 18: _t->slotUpdateSignature(); break;
        case 19: _t->updateWindowTitle(); break;
        case 20: _t->autoSaveDraft(); break;
        case 21: _t->setMessageUpdated(); break;
        case 22: _t->setUiWidgetsEnabled((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 23: _t->onCompletionAvailable((*reinterpret_cast< const Plugins::NameEmailList(*)>(_a[1]))); break;
        case 24: _t->onCompletionFailed((*reinterpret_cast< Plugins::AddressbookJob::Error(*)>(_a[1]))); break;
        case 25: _t->passwordRequested((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 26: _t->passwordError(); break;
        case 27: _t->toggleReplyMarking(); break;
        case 28: _t->updateReplyMarkingAction(); break;
        case 29: _t->updateReplyMode(); break;
        case 30: _t->markReplyModeHandpicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ComposeWidget::*)(const Common::LogKind , const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComposeWidget::logged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::ComposeWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Gui__ComposeWidget.data,
    qt_meta_data_Gui__ComposeWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::ComposeWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::ComposeWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__ComposeWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Gui::ComposeWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    return _id;
}

// SIGNAL 0
void Gui::ComposeWidget::logged(const Common::LogKind _t1, const QString & _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
