/********************************************************************************
** Form generated from reading UI file 'EditFavoriteTag.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITFAVORITETAG_H
#define UI_EDITFAVORITETAG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>
#include "LineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_EditFavoriteTag
{
public:
    QFormLayout *formLayout;
    QLabel *nameLabel;
    LineEdit *nameLineEdit;
    QLabel *colorLabel;
    QDialogButtonBox *buttonBox;
    QPushButton *colorButton;

    void setupUi(QWidget *EditFavoriteTag)
    {
        if (EditFavoriteTag->objectName().isEmpty())
            EditFavoriteTag->setObjectName(QString::fromUtf8("EditFavoriteTag"));
        EditFavoriteTag->resize(250, 116);
        EditFavoriteTag->setMinimumSize(QSize(0, 0));
        formLayout = new QFormLayout(EditFavoriteTag);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        nameLabel = new QLabel(EditFavoriteTag);
        nameLabel->setObjectName(QString::fromUtf8("nameLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, nameLabel);

        nameLineEdit = new LineEdit(EditFavoriteTag);
        nameLineEdit->setObjectName(QString::fromUtf8("nameLineEdit"));

        formLayout->setWidget(0, QFormLayout::FieldRole, nameLineEdit);

        colorLabel = new QLabel(EditFavoriteTag);
        colorLabel->setObjectName(QString::fromUtf8("colorLabel"));

        formLayout->setWidget(1, QFormLayout::LabelRole, colorLabel);

        buttonBox = new QDialogButtonBox(EditFavoriteTag);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        formLayout->setWidget(2, QFormLayout::FieldRole, buttonBox);

        colorButton = new QPushButton(EditFavoriteTag);
        colorButton->setObjectName(QString::fromUtf8("colorButton"));

        formLayout->setWidget(1, QFormLayout::FieldRole, colorButton);


        retranslateUi(EditFavoriteTag);

        QMetaObject::connectSlotsByName(EditFavoriteTag);
    } // setupUi

    void retranslateUi(QWidget *EditFavoriteTag)
    {
        EditFavoriteTag->setWindowTitle(QCoreApplication::translate("EditFavoriteTag", "Manage Tag", nullptr));
        nameLabel->setText(QCoreApplication::translate("EditFavoriteTag", "Name", nullptr));
        colorLabel->setText(QCoreApplication::translate("EditFavoriteTag", "Color", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditFavoriteTag: public Ui_EditFavoriteTag {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITFAVORITETAG_H
