/********************************************************************************
** Form generated from reading UI file 'SettingsOutgoingPage.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGSOUTGOINGPAGE_H
#define UI_SETTINGSOUTGOINGPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QWidget>
#include "LineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_OutgoingPage
{
public:
    QWidget *scrollAreaWidgetContents;
    QFormLayout *formLayout;
    QLabel *methodLabel;
    QComboBox *method;
    QLabel *encryptionLabel;
    QComboBox *encryption;
    QLabel *smtpHostLabel;
    LineEdit *smtpHost;
    QLabel *smtpPortLabel;
    LineEdit *smtpPort;
    QLabel *portWarningLabel;
    QLabel *smtpAuthLabel;
    QCheckBox *smtpAuth;
    QLabel *smtpUserLabel;
    LineEdit *smtpUser;
    QLabel *smtpPassLabel;
    LineEdit *smtpPass;
    QLabel *passwordWarning;
    QLabel *passwordPluginStatus;
    QLabel *sendmailLabel;
    LineEdit *sendmail;
    QLabel *saveToImapLabel;
    QCheckBox *saveToImap;
    QLabel *saveFolderLabel;
    LineEdit *saveFolderName;
    QLabel *smtpBurlLabel;
    QCheckBox *smtpBurl;
    QCheckBox *smtpAuthReuseImapCreds;
    QLabel *smtpAuthReuseImapCredsLabel;

    void setupUi(QScrollArea *OutgoingPage)
    {
        if (OutgoingPage->objectName().isEmpty())
            OutgoingPage->setObjectName(QString::fromUtf8("OutgoingPage"));
        OutgoingPage->setFrameShadow(QFrame::Plain);
        OutgoingPage->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 220, 347));
        formLayout = new QFormLayout(scrollAreaWidgetContents);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        methodLabel = new QLabel(scrollAreaWidgetContents);
        methodLabel->setObjectName(QString::fromUtf8("methodLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, methodLabel);

        method = new QComboBox(scrollAreaWidgetContents);
        method->setObjectName(QString::fromUtf8("method"));

        formLayout->setWidget(0, QFormLayout::FieldRole, method);

        encryptionLabel = new QLabel(scrollAreaWidgetContents);
        encryptionLabel->setObjectName(QString::fromUtf8("encryptionLabel"));

        formLayout->setWidget(1, QFormLayout::LabelRole, encryptionLabel);

        encryption = new QComboBox(scrollAreaWidgetContents);
        encryption->setObjectName(QString::fromUtf8("encryption"));

        formLayout->setWidget(1, QFormLayout::FieldRole, encryption);

        smtpHostLabel = new QLabel(scrollAreaWidgetContents);
        smtpHostLabel->setObjectName(QString::fromUtf8("smtpHostLabel"));

        formLayout->setWidget(2, QFormLayout::LabelRole, smtpHostLabel);

        smtpHost = new LineEdit(scrollAreaWidgetContents);
        smtpHost->setObjectName(QString::fromUtf8("smtpHost"));

        formLayout->setWidget(2, QFormLayout::FieldRole, smtpHost);

        smtpPortLabel = new QLabel(scrollAreaWidgetContents);
        smtpPortLabel->setObjectName(QString::fromUtf8("smtpPortLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, smtpPortLabel);

        smtpPort = new LineEdit(scrollAreaWidgetContents);
        smtpPort->setObjectName(QString::fromUtf8("smtpPort"));

        formLayout->setWidget(3, QFormLayout::FieldRole, smtpPort);

        portWarningLabel = new QLabel(scrollAreaWidgetContents);
        portWarningLabel->setObjectName(QString::fromUtf8("portWarningLabel"));
        portWarningLabel->setWordWrap(true);

        formLayout->setWidget(4, QFormLayout::SpanningRole, portWarningLabel);

        smtpAuthLabel = new QLabel(scrollAreaWidgetContents);
        smtpAuthLabel->setObjectName(QString::fromUtf8("smtpAuthLabel"));

        formLayout->setWidget(5, QFormLayout::LabelRole, smtpAuthLabel);

        smtpAuth = new QCheckBox(scrollAreaWidgetContents);
        smtpAuth->setObjectName(QString::fromUtf8("smtpAuth"));

        formLayout->setWidget(5, QFormLayout::FieldRole, smtpAuth);

        smtpUserLabel = new QLabel(scrollAreaWidgetContents);
        smtpUserLabel->setObjectName(QString::fromUtf8("smtpUserLabel"));

        formLayout->setWidget(7, QFormLayout::LabelRole, smtpUserLabel);

        smtpUser = new LineEdit(scrollAreaWidgetContents);
        smtpUser->setObjectName(QString::fromUtf8("smtpUser"));

        formLayout->setWidget(7, QFormLayout::FieldRole, smtpUser);

        smtpPassLabel = new QLabel(scrollAreaWidgetContents);
        smtpPassLabel->setObjectName(QString::fromUtf8("smtpPassLabel"));

        formLayout->setWidget(9, QFormLayout::LabelRole, smtpPassLabel);

        smtpPass = new LineEdit(scrollAreaWidgetContents);
        smtpPass->setObjectName(QString::fromUtf8("smtpPass"));
        smtpPass->setEchoMode(QLineEdit::Password);

        formLayout->setWidget(9, QFormLayout::FieldRole, smtpPass);

        passwordWarning = new QLabel(scrollAreaWidgetContents);
        passwordWarning->setObjectName(QString::fromUtf8("passwordWarning"));
        passwordWarning->setWordWrap(true);

        formLayout->setWidget(10, QFormLayout::SpanningRole, passwordWarning);

        passwordPluginStatus = new QLabel(scrollAreaWidgetContents);
        passwordPluginStatus->setObjectName(QString::fromUtf8("passwordPluginStatus"));
        passwordPluginStatus->setText(QString::fromUtf8("plugin status"));
        passwordPluginStatus->setAlignment(Qt::AlignCenter);
        passwordPluginStatus->setWordWrap(true);
        passwordPluginStatus->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout->setWidget(11, QFormLayout::SpanningRole, passwordPluginStatus);

        sendmailLabel = new QLabel(scrollAreaWidgetContents);
        sendmailLabel->setObjectName(QString::fromUtf8("sendmailLabel"));

        formLayout->setWidget(12, QFormLayout::LabelRole, sendmailLabel);

        sendmail = new LineEdit(scrollAreaWidgetContents);
        sendmail->setObjectName(QString::fromUtf8("sendmail"));

        formLayout->setWidget(12, QFormLayout::FieldRole, sendmail);

        saveToImapLabel = new QLabel(scrollAreaWidgetContents);
        saveToImapLabel->setObjectName(QString::fromUtf8("saveToImapLabel"));

        formLayout->setWidget(13, QFormLayout::LabelRole, saveToImapLabel);

        saveToImap = new QCheckBox(scrollAreaWidgetContents);
        saveToImap->setObjectName(QString::fromUtf8("saveToImap"));

        formLayout->setWidget(13, QFormLayout::FieldRole, saveToImap);

        saveFolderLabel = new QLabel(scrollAreaWidgetContents);
        saveFolderLabel->setObjectName(QString::fromUtf8("saveFolderLabel"));

        formLayout->setWidget(14, QFormLayout::LabelRole, saveFolderLabel);

        saveFolderName = new LineEdit(scrollAreaWidgetContents);
        saveFolderName->setObjectName(QString::fromUtf8("saveFolderName"));

        formLayout->setWidget(14, QFormLayout::FieldRole, saveFolderName);

        smtpBurlLabel = new QLabel(scrollAreaWidgetContents);
        smtpBurlLabel->setObjectName(QString::fromUtf8("smtpBurlLabel"));

        formLayout->setWidget(15, QFormLayout::LabelRole, smtpBurlLabel);

        smtpBurl = new QCheckBox(scrollAreaWidgetContents);
        smtpBurl->setObjectName(QString::fromUtf8("smtpBurl"));

        formLayout->setWidget(15, QFormLayout::FieldRole, smtpBurl);

        smtpAuthReuseImapCreds = new QCheckBox(scrollAreaWidgetContents);
        smtpAuthReuseImapCreds->setObjectName(QString::fromUtf8("smtpAuthReuseImapCreds"));

        formLayout->setWidget(6, QFormLayout::FieldRole, smtpAuthReuseImapCreds);

        smtpAuthReuseImapCredsLabel = new QLabel(scrollAreaWidgetContents);
        smtpAuthReuseImapCredsLabel->setObjectName(QString::fromUtf8("smtpAuthReuseImapCredsLabel"));

        formLayout->setWidget(6, QFormLayout::LabelRole, smtpAuthReuseImapCredsLabel);

        OutgoingPage->setWidget(scrollAreaWidgetContents);
#if QT_CONFIG(shortcut)
        methodLabel->setBuddy(method);
        smtpHostLabel->setBuddy(smtpHost);
        smtpPortLabel->setBuddy(smtpPort);
        smtpAuthLabel->setBuddy(smtpAuth);
        smtpUserLabel->setBuddy(smtpUser);
        smtpPassLabel->setBuddy(smtpPass);
        sendmailLabel->setBuddy(sendmail);
        saveToImapLabel->setBuddy(saveToImap);
        saveFolderLabel->setBuddy(saveFolderName);
        smtpBurlLabel->setBuddy(smtpBurl);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(method, smtpHost);
        QWidget::setTabOrder(smtpHost, smtpPort);
        QWidget::setTabOrder(smtpPort, smtpAuth);
        QWidget::setTabOrder(smtpAuth, smtpUser);
        QWidget::setTabOrder(smtpUser, smtpPass);
        QWidget::setTabOrder(smtpPass, sendmail);
        QWidget::setTabOrder(sendmail, saveToImap);
        QWidget::setTabOrder(saveToImap, saveFolderName);
        QWidget::setTabOrder(saveFolderName, smtpBurl);

        retranslateUi(OutgoingPage);

        QMetaObject::connectSlotsByName(OutgoingPage);
    } // setupUi

    void retranslateUi(QScrollArea *OutgoingPage)
    {
        methodLabel->setText(QCoreApplication::translate("OutgoingPage", "&Method", nullptr));
        encryptionLabel->setText(QCoreApplication::translate("OutgoingPage", "Encryption", nullptr));
        smtpHostLabel->setText(QCoreApplication::translate("OutgoingPage", "Ser&ver", nullptr));
        smtpPortLabel->setText(QCoreApplication::translate("OutgoingPage", "&Port", nullptr));
        portWarningLabel->setText(QString());
        smtpAuthLabel->setText(QCoreApplication::translate("OutgoingPage", "&Authenticate", nullptr));
        smtpAuth->setText(QString());
        smtpUserLabel->setText(QCoreApplication::translate("OutgoingPage", "&Username", nullptr));
        smtpPassLabel->setText(QCoreApplication::translate("OutgoingPage", "Pass&word", nullptr));
        sendmailLabel->setText(QCoreApplication::translate("OutgoingPage", "Sendmail E&xecutable", nullptr));
        saveToImapLabel->setText(QCoreApplication::translate("OutgoingPage", "Save Ou&tgoing Mail", nullptr));
        saveFolderLabel->setText(QCoreApplication::translate("OutgoingPage", "Sent Folder &Name", nullptr));
#if QT_CONFIG(tooltip)
        smtpBurlLabel->setToolTip(QCoreApplication::translate("OutgoingPage", "Enable forward-without-download via BURL (RFC 4468) when the IMAP and ESMTP servers support it. Extra server-side configuration is required.", nullptr));
#endif // QT_CONFIG(tooltip)
        smtpBurlLabel->setText(QCoreApplication::translate("OutgoingPage", "Send with &BURL", nullptr));
        smtpAuthReuseImapCreds->setText(QString());
        smtpAuthReuseImapCredsLabel->setText(QCoreApplication::translate("OutgoingPage", "Use IMAP Credentials", nullptr));
        (void)OutgoingPage;
    } // retranslateUi

};

namespace Ui {
    class OutgoingPage: public Ui_OutgoingPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGSOUTGOINGPAGE_H
