/********************************************************************************
** Form generated from reading UI file 'SettingsFavoriteTagsPage.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGSFAVORITETAGSPAGE_H
#define UI_SETTINGSFAVORITETAGSPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FavoriteTagsPage
{
public:
    QWidget *scrollAreaWidgetContents;
    QGridLayout *gridLayout;
    QTableView *tagTableView;
    QVBoxLayout *verticalLayout;
    QPushButton *moveUpButton;
    QPushButton *moveDownButton;
    QFrame *line;
    QHBoxLayout *horizontalLayout;
    QPushButton *addButton;
    QPushButton *editButton;
    QPushButton *deleteButton;
    QSpacerItem *verticalSpacer;

    void setupUi(QScrollArea *FavoriteTagsPage)
    {
        if (FavoriteTagsPage->objectName().isEmpty())
            FavoriteTagsPage->setObjectName(QString::fromUtf8("FavoriteTagsPage"));
        FavoriteTagsPage->resize(550, 716);
        FavoriteTagsPage->setFrameShadow(QFrame::Plain);
        FavoriteTagsPage->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 546, 712));
        scrollAreaWidgetContents->setLayoutDirection(Qt::LeftToRight);
        gridLayout = new QGridLayout(scrollAreaWidgetContents);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        tagTableView = new QTableView(scrollAreaWidgetContents);
        tagTableView->setObjectName(QString::fromUtf8("tagTableView"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(tagTableView->sizePolicy().hasHeightForWidth());
        tagTableView->setSizePolicy(sizePolicy);
        tagTableView->setMinimumSize(QSize(241, 192));

        gridLayout->addWidget(tagTableView, 0, 0, 2, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        moveUpButton = new QPushButton(scrollAreaWidgetContents);
        moveUpButton->setObjectName(QString::fromUtf8("moveUpButton"));
        moveUpButton->setMinimumSize(QSize(29, 29));
        moveUpButton->setMaximumSize(QSize(29, 29));

        verticalLayout->addWidget(moveUpButton);

        moveDownButton = new QPushButton(scrollAreaWidgetContents);
        moveDownButton->setObjectName(QString::fromUtf8("moveDownButton"));
        moveDownButton->setMinimumSize(QSize(29, 29));
        moveDownButton->setMaximumSize(QSize(29, 29));

        verticalLayout->addWidget(moveDownButton);


        gridLayout->addLayout(verticalLayout, 0, 1, 1, 1);

        line = new QFrame(scrollAreaWidgetContents);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line, 3, 0, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetFixedSize);
        addButton = new QPushButton(scrollAreaWidgetContents);
        addButton->setObjectName(QString::fromUtf8("addButton"));

        horizontalLayout->addWidget(addButton);

        editButton = new QPushButton(scrollAreaWidgetContents);
        editButton->setObjectName(QString::fromUtf8("editButton"));

        horizontalLayout->addWidget(editButton);

        deleteButton = new QPushButton(scrollAreaWidgetContents);
        deleteButton->setObjectName(QString::fromUtf8("deleteButton"));

        horizontalLayout->addWidget(deleteButton);


        gridLayout->addLayout(horizontalLayout, 2, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 125, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(verticalSpacer, 1, 1, 2, 1);

        FavoriteTagsPage->setWidget(scrollAreaWidgetContents);

        retranslateUi(FavoriteTagsPage);

        QMetaObject::connectSlotsByName(FavoriteTagsPage);
    } // setupUi

    void retranslateUi(QScrollArea *FavoriteTagsPage)
    {
        FavoriteTagsPage->setWindowTitle(QCoreApplication::translate("FavoriteTagsPage", "ScrollArea", nullptr));
        moveUpButton->setText(QString());
        moveDownButton->setText(QString());
        addButton->setText(QCoreApplication::translate("FavoriteTagsPage", "Add", nullptr));
        editButton->setText(QCoreApplication::translate("FavoriteTagsPage", "Edit", nullptr));
        deleteButton->setText(QCoreApplication::translate("FavoriteTagsPage", "Delete", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FavoriteTagsPage: public Ui_FavoriteTagsPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGSFAVORITETAGSPAGE_H
