/****************************************************************************
** Meta object code from reading C++ file 'SMTP.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/MSA/SMTP.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SMTP.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MSA__SMTP_t {
    QByteArrayData data[14];
    char stringdata0[142];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MSA__SMTP_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MSA__SMTP_t qt_meta_stringdata_MSA__SMTP = {
    {
QT_MOC_LITERAL(0, 0, 9), // "MSA::SMTP"
QT_MOC_LITERAL(1, 10, 6), // "cancel"
QT_MOC_LITERAL(2, 17, 0), // ""
QT_MOC_LITERAL(3, 18, 11), // "setPassword"
QT_MOC_LITERAL(4, 30, 8), // "password"
QT_MOC_LITERAL(5, 39, 10), // "handleDone"
QT_MOC_LITERAL(6, 50, 2), // "ok"
QT_MOC_LITERAL(7, 53, 11), // "handleError"
QT_MOC_LITERAL(8, 65, 28), // "QAbstractSocket::SocketError"
QT_MOC_LITERAL(9, 94, 3), // "err"
QT_MOC_LITERAL(10, 98, 3), // "msg"
QT_MOC_LITERAL(11, 102, 15), // "handleSslErrors"
QT_MOC_LITERAL(12, 118, 16), // "QList<QSslError>"
QT_MOC_LITERAL(13, 135, 6) // "errors"

    },
    "MSA::SMTP\0cancel\0\0setPassword\0password\0"
    "handleDone\0ok\0handleError\0"
    "QAbstractSocket::SocketError\0err\0msg\0"
    "handleSslErrors\0QList<QSslError>\0"
    "errors"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MSA__SMTP[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x0a /* Public */,
       3,    1,   40,    2, 0x0a /* Public */,
       5,    1,   43,    2, 0x0a /* Public */,
       7,    2,   46,    2, 0x0a /* Public */,
      11,    1,   51,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, 0x80000000 | 8, QMetaType::QString,    9,   10,
    QMetaType::Void, 0x80000000 | 12,   13,

       0        // eod
};

void MSA::SMTP::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SMTP *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cancel(); break;
        case 1: _t->setPassword((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->handleDone((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->handleError((*reinterpret_cast< QAbstractSocket::SocketError(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: _t->handleSslErrors((*reinterpret_cast< const QList<QSslError>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractSocket::SocketError >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QSslError> >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MSA::SMTP::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractMSA::staticMetaObject>(),
    qt_meta_stringdata_MSA__SMTP.data,
    qt_meta_data_MSA__SMTP,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MSA::SMTP::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MSA::SMTP::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MSA__SMTP.stringdata0))
        return static_cast<void*>(this);
    return AbstractMSA::qt_metacast(_clname);
}

int MSA::SMTP::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractMSA::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
