/****************************************************************************
** Meta object code from reading C++ file 'IODeviceSocket.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Streams/IODeviceSocket.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'IODeviceSocket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Streams__IODeviceSocket_t {
    QByteArrayData data[6];
    char stringdata0[83];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Streams__IODeviceSocket_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Streams__IODeviceSocket_t qt_meta_stringdata_Streams__IODeviceSocket = {
    {
QT_MOC_LITERAL(0, 0, 23), // "Streams::IODeviceSocket"
QT_MOC_LITERAL(1, 24, 18), // "handleStateChanged"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 12), // "delayedStart"
QT_MOC_LITERAL(4, 57, 15), // "handleReadyRead"
QT_MOC_LITERAL(5, 73, 9) // "emitError"

    },
    "Streams::IODeviceSocket\0handleStateChanged\0"
    "\0delayedStart\0handleReadyRead\0emitError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Streams__IODeviceSocket[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x08 /* Private */,
       3,    0,   35,    2, 0x08 /* Private */,
       4,    0,   36,    2, 0x08 /* Private */,
       5,    0,   37,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Streams::IODeviceSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<IODeviceSocket *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleStateChanged(); break;
        case 1: _t->delayedStart(); break;
        case 2: _t->handleReadyRead(); break;
        case 3: _t->emitError(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Streams::IODeviceSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<Socket::staticMetaObject>(),
    qt_meta_stringdata_Streams__IODeviceSocket.data,
    qt_meta_data_Streams__IODeviceSocket,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Streams::IODeviceSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Streams::IODeviceSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Streams__IODeviceSocket.stringdata0))
        return static_cast<void*>(this);
    return Socket::qt_metacast(_clname);
}

int Streams::IODeviceSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Socket::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_Streams__ProcessSocket_t {
    QByteArrayData data[6];
    char stringdata0[98];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Streams__ProcessSocket_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Streams__ProcessSocket_t qt_meta_stringdata_Streams__ProcessSocket = {
    {
QT_MOC_LITERAL(0, 0, 22), // "Streams::ProcessSocket"
QT_MOC_LITERAL(1, 23, 18), // "handleStateChanged"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 18), // "handleProcessError"
QT_MOC_LITERAL(4, 62, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(5, 85, 12) // "delayedStart"

    },
    "Streams::ProcessSocket\0handleStateChanged\0"
    "\0handleProcessError\0QProcess::ProcessError\0"
    "delayedStart"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Streams__ProcessSocket[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x08 /* Private */,
       3,    1,   30,    2, 0x08 /* Private */,
       5,    0,   33,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,

       0        // eod
};

void Streams::ProcessSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ProcessSocket *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleStateChanged(); break;
        case 1: _t->handleProcessError((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 2: _t->delayedStart(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Streams::ProcessSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<IODeviceSocket::staticMetaObject>(),
    qt_meta_stringdata_Streams__ProcessSocket.data,
    qt_meta_data_Streams__ProcessSocket,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Streams::ProcessSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Streams::ProcessSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Streams__ProcessSocket.stringdata0))
        return static_cast<void*>(this);
    return IODeviceSocket::qt_metacast(_clname);
}

int Streams::ProcessSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = IODeviceSocket::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_Streams__SslTlsSocket_t {
    QByteArrayData data[6];
    char stringdata0[102];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Streams__SslTlsSocket_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Streams__SslTlsSocket_t qt_meta_stringdata_Streams__SslTlsSocket = {
    {
QT_MOC_LITERAL(0, 0, 21), // "Streams::SslTlsSocket"
QT_MOC_LITERAL(1, 22, 18), // "handleStateChanged"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 17), // "handleSocketError"
QT_MOC_LITERAL(4, 60, 28), // "QAbstractSocket::SocketError"
QT_MOC_LITERAL(5, 89, 12) // "delayedStart"

    },
    "Streams::SslTlsSocket\0handleStateChanged\0"
    "\0handleSocketError\0QAbstractSocket::SocketError\0"
    "delayedStart"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Streams__SslTlsSocket[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x08 /* Private */,
       3,    1,   30,    2, 0x08 /* Private */,
       5,    0,   33,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,

       0        // eod
};

void Streams::SslTlsSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SslTlsSocket *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleStateChanged(); break;
        case 1: _t->handleSocketError((*reinterpret_cast< QAbstractSocket::SocketError(*)>(_a[1]))); break;
        case 2: _t->delayedStart(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractSocket::SocketError >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Streams::SslTlsSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<IODeviceSocket::staticMetaObject>(),
    qt_meta_stringdata_Streams__SslTlsSocket.data,
    qt_meta_data_Streams__SslTlsSocket,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Streams::SslTlsSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Streams::SslTlsSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Streams__SslTlsSocket.stringdata0))
        return static_cast<void*>(this);
    return IODeviceSocket::qt_metacast(_clname);
}

int Streams::SslTlsSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = IODeviceSocket::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
