/****************************************************************************
** Meta object code from reading C++ file 'Model.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/Model.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Model.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Mailbox__Model_t {
    QByteArrayData data[80];
    char stringdata0[1309];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__Model_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__Model_t qt_meta_stringdata_Imap__Mailbox__Model = {
    {
QT_MOC_LITERAL(0, 0, 20), // "Imap::Mailbox::Model"
QT_MOC_LITERAL(1, 21, 13), // "alertReceived"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 7), // "message"
QT_MOC_LITERAL(4, 44, 20), // "networkPolicyOffline"
QT_MOC_LITERAL(5, 65, 22), // "networkPolicyExpensive"
QT_MOC_LITERAL(6, 88, 19), // "networkPolicyOnline"
QT_MOC_LITERAL(7, 108, 20), // "networkPolicyChanged"
QT_MOC_LITERAL(8, 129, 12), // "networkError"
QT_MOC_LITERAL(9, 142, 9), // "imapError"
QT_MOC_LITERAL(10, 152, 13), // "authRequested"
QT_MOC_LITERAL(11, 166, 17), // "authAttemptFailed"
QT_MOC_LITERAL(12, 184, 16), // "needsSslDecision"
QT_MOC_LITERAL(13, 201, 22), // "QList<QSslCertificate>"
QT_MOC_LITERAL(14, 224, 12), // "certificates"
QT_MOC_LITERAL(15, 237, 16), // "QList<QSslError>"
QT_MOC_LITERAL(16, 254, 9), // "sslErrors"
QT_MOC_LITERAL(17, 264, 23), // "requireStartTlsInFuture"
QT_MOC_LITERAL(18, 288, 27), // "messageCountPossiblyChanged"
QT_MOC_LITERAL(19, 316, 11), // "QModelIndex"
QT_MOC_LITERAL(20, 328, 7), // "mailbox"
QT_MOC_LITERAL(21, 336, 23), // "mailboxCreationSucceded"
QT_MOC_LITERAL(22, 360, 21), // "mailboxCreationFailed"
QT_MOC_LITERAL(23, 382, 23), // "mailboxDeletionSucceded"
QT_MOC_LITERAL(24, 406, 21), // "mailboxDeletionFailed"
QT_MOC_LITERAL(25, 428, 17), // "mailboxSyncFailed"
QT_MOC_LITERAL(26, 446, 22), // "connectionStateChanged"
QT_MOC_LITERAL(27, 469, 8), // "parserId"
QT_MOC_LITERAL(28, 478, 21), // "Imap::ConnectionState"
QT_MOC_LITERAL(29, 500, 5), // "state"
QT_MOC_LITERAL(30, 506, 19), // "logParserFatalError"
QT_MOC_LITERAL(31, 526, 6), // "parser"
QT_MOC_LITERAL(32, 533, 14), // "exceptionClass"
QT_MOC_LITERAL(33, 548, 4), // "line"
QT_MOC_LITERAL(34, 553, 8), // "position"
QT_MOC_LITERAL(35, 562, 22), // "mailboxSyncingProgress"
QT_MOC_LITERAL(36, 585, 37), // "Imap::Mailbox::MailboxSyncing..."
QT_MOC_LITERAL(37, 623, 25), // "mailboxFirstUnseenMessage"
QT_MOC_LITERAL(38, 649, 8), // "maillbox"
QT_MOC_LITERAL(39, 658, 18), // "threadingAvailable"
QT_MOC_LITERAL(40, 677, 9), // "algorithm"
QT_MOC_LITERAL(41, 687, 14), // "searchCriteria"
QT_MOC_LITERAL(42, 702, 39), // "QVector<Imap::Responses::Thre..."
QT_MOC_LITERAL(43, 742, 7), // "mapping"
QT_MOC_LITERAL(44, 750, 15), // "threadingFailed"
QT_MOC_LITERAL(45, 766, 19), // "capabilitiesUpdated"
QT_MOC_LITERAL(46, 786, 12), // "capabilities"
QT_MOC_LITERAL(47, 799, 6), // "logged"
QT_MOC_LITERAL(48, 806, 18), // "Common::LogMessage"
QT_MOC_LITERAL(49, 825, 20), // "imapAuthErrorChanged"
QT_MOC_LITERAL(50, 846, 5), // "error"
QT_MOC_LITERAL(51, 852, 17), // "reloadMailboxList"
QT_MOC_LITERAL(52, 870, 15), // "switchToMailbox"
QT_MOC_LITERAL(53, 886, 4), // "mbox"
QT_MOC_LITERAL(54, 891, 9), // "taskModel"
QT_MOC_LITERAL(55, 901, 19), // "QAbstractItemModel*"
QT_MOC_LITERAL(56, 921, 12), // "setSslPolicy"
QT_MOC_LITERAL(57, 934, 8), // "sslChain"
QT_MOC_LITERAL(58, 943, 7), // "proceed"
QT_MOC_LITERAL(59, 951, 26), // "invalidateAllMessageCounts"
QT_MOC_LITERAL(60, 978, 13), // "imapAuthError"
QT_MOC_LITERAL(61, 992, 24), // "handleSocketStateChanged"
QT_MOC_LITERAL(62, 1017, 13), // "Imap::Parser*"
QT_MOC_LITERAL(63, 1031, 22), // "slotParserLineReceived"
QT_MOC_LITERAL(64, 1054, 18), // "slotParserLineSent"
QT_MOC_LITERAL(65, 1073, 16), // "slotTasksChanged"
QT_MOC_LITERAL(66, 1090, 13), // "slotTaskDying"
QT_MOC_LITERAL(67, 1104, 3), // "obj"
QT_MOC_LITERAL(68, 1108, 16), // "setImapAuthError"
QT_MOC_LITERAL(69, 1125, 16), // "responseReceived"
QT_MOC_LITERAL(70, 1142, 23), // "askForChildrenOfMailbox"
QT_MOC_LITERAL(71, 1166, 5), // "index"
QT_MOC_LITERAL(72, 1172, 31), // "Imap::Mailbox::CacheLoadingMode"
QT_MOC_LITERAL(73, 1204, 9), // "cacheMode"
QT_MOC_LITERAL(74, 1214, 23), // "askForMessagesInMailbox"
QT_MOC_LITERAL(75, 1238, 13), // "runReadyTasks"
QT_MOC_LITERAL(76, 1252, 8), // "imapUser"
QT_MOC_LITERAL(77, 1261, 12), // "imapPassword"
QT_MOC_LITERAL(78, 1274, 18), // "isNetworkAvailable"
QT_MOC_LITERAL(79, 1293, 15) // "isNetworkOnline"

    },
    "Imap::Mailbox::Model\0alertReceived\0\0"
    "message\0networkPolicyOffline\0"
    "networkPolicyExpensive\0networkPolicyOnline\0"
    "networkPolicyChanged\0networkError\0"
    "imapError\0authRequested\0authAttemptFailed\0"
    "needsSslDecision\0QList<QSslCertificate>\0"
    "certificates\0QList<QSslError>\0sslErrors\0"
    "requireStartTlsInFuture\0"
    "messageCountPossiblyChanged\0QModelIndex\0"
    "mailbox\0mailboxCreationSucceded\0"
    "mailboxCreationFailed\0mailboxDeletionSucceded\0"
    "mailboxDeletionFailed\0mailboxSyncFailed\0"
    "connectionStateChanged\0parserId\0"
    "Imap::ConnectionState\0state\0"
    "logParserFatalError\0parser\0exceptionClass\0"
    "line\0position\0mailboxSyncingProgress\0"
    "Imap::Mailbox::MailboxSyncingProgress\0"
    "mailboxFirstUnseenMessage\0maillbox\0"
    "threadingAvailable\0algorithm\0"
    "searchCriteria\0QVector<Imap::Responses::ThreadingNode>\0"
    "mapping\0threadingFailed\0capabilitiesUpdated\0"
    "capabilities\0logged\0Common::LogMessage\0"
    "imapAuthErrorChanged\0error\0reloadMailboxList\0"
    "switchToMailbox\0mbox\0taskModel\0"
    "QAbstractItemModel*\0setSslPolicy\0"
    "sslChain\0proceed\0invalidateAllMessageCounts\0"
    "imapAuthError\0handleSocketStateChanged\0"
    "Imap::Parser*\0slotParserLineReceived\0"
    "slotParserLineSent\0slotTasksChanged\0"
    "slotTaskDying\0obj\0setImapAuthError\0"
    "responseReceived\0askForChildrenOfMailbox\0"
    "index\0Imap::Mailbox::CacheLoadingMode\0"
    "cacheMode\0askForMessagesInMailbox\0"
    "runReadyTasks\0imapUser\0imapPassword\0"
    "isNetworkAvailable\0isNetworkOnline"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__Model[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      43,   14, // methods
       5,  378, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      26,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  229,    2, 0x06 /* Public */,
       4,    0,  232,    2, 0x06 /* Public */,
       5,    0,  233,    2, 0x06 /* Public */,
       6,    0,  234,    2, 0x06 /* Public */,
       7,    0,  235,    2, 0x06 /* Public */,
       8,    1,  236,    2, 0x06 /* Public */,
       9,    1,  239,    2, 0x06 /* Public */,
      10,    0,  242,    2, 0x06 /* Public */,
      11,    1,  243,    2, 0x06 /* Public */,
      12,    2,  246,    2, 0x06 /* Public */,
      17,    0,  251,    2, 0x06 /* Public */,
      18,    1,  252,    2, 0x06 /* Public */,
      21,    1,  255,    2, 0x06 /* Public */,
      22,    2,  258,    2, 0x06 /* Public */,
      23,    1,  263,    2, 0x06 /* Public */,
      24,    2,  266,    2, 0x06 /* Public */,
      25,    2,  271,    2, 0x06 /* Public */,
      26,    2,  276,    2, 0x06 /* Public */,
      30,    5,  281,    2, 0x06 /* Public */,
      35,    2,  292,    2, 0x06 /* Public */,
      37,    2,  297,    2, 0x06 /* Public */,
      39,    4,  302,    2, 0x06 /* Public */,
      44,    3,  311,    2, 0x06 /* Public */,
      45,    1,  318,    2, 0x06 /* Public */,
      47,    2,  321,    2, 0x06 /* Public */,
      49,    1,  326,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      51,    0,  329,    2, 0x0a /* Public */,
      52,    1,  330,    2, 0x0a /* Public */,
      54,    0,  333,    2, 0x0a /* Public */,
      56,    3,  334,    2, 0x0a /* Public */,
      59,    0,  341,    2, 0x0a /* Public */,
      60,    0,  342,    2, 0x0a /* Public */,
      61,    2,  343,    2, 0x08 /* Private */,
      63,    2,  348,    2, 0x08 /* Private */,
      64,    2,  353,    2, 0x08 /* Private */,
      65,    0,  358,    2, 0x08 /* Private */,
      66,    1,  359,    2, 0x08 /* Private */,
      68,    1,  362,    2, 0x08 /* Private */,
      69,    0,  365,    2, 0x09 /* Protected */,
      69,    1,  366,    2, 0x09 /* Protected */,
      70,    2,  369,    2, 0x09 /* Protected */,
      74,    1,  374,    2, 0x09 /* Protected */,
      75,    0,  377,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, 0x80000000 | 13, 0x80000000 | 15,   14,   16,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 19,   20,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   20,    3,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   20,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   20,    3,
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 28,   27,   29,
    QMetaType::Void, QMetaType::UInt, QMetaType::QString, QMetaType::QString, QMetaType::QByteArray, QMetaType::Int,   31,   32,    3,   33,   34,
    QMetaType::Void, 0x80000000 | 19, 0x80000000 | 36,   20,   29,
    QMetaType::Void, 0x80000000 | 19, 0x80000000 | 19,   38,    3,
    QMetaType::Void, 0x80000000 | 19, QMetaType::QByteArray, QMetaType::QStringList, 0x80000000 | 42,   20,   40,   41,   43,
    QMetaType::Void, 0x80000000 | 19, QMetaType::QByteArray, QMetaType::QStringList,   20,   40,   41,
    QMetaType::Void, QMetaType::QStringList,   46,
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 48,   27,    3,
    QMetaType::Void, QMetaType::QString,   50,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 19,   53,
    0x80000000 | 55,
    QMetaType::Void, 0x80000000 | 13, 0x80000000 | 15, QMetaType::Bool,   57,   16,   58,
    QMetaType::Void,
    QMetaType::QString,
    QMetaType::Void, 0x80000000 | 62, 0x80000000 | 28,   31,   29,
    QMetaType::Void, 0x80000000 | 62, QMetaType::QByteArray,   31,   33,
    QMetaType::Void, 0x80000000 | 62, QMetaType::QByteArray,   31,   33,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QObjectStar,   67,
    QMetaType::Void, QMetaType::QString,   50,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 62,   31,
    QMetaType::Void, 0x80000000 | 19, 0x80000000 | 72,   71,   73,
    QMetaType::Void, 0x80000000 | 19,   71,
    QMetaType::Void,

 // properties: name, type, flags
      76, QMetaType::QString, 0x00095103,
      77, QMetaType::QString, 0x00095107,
      60, QMetaType::QString, 0x00495001,
      78, QMetaType::Bool, 0x00495001,
      79, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
      25,
       4,
       4,

       0        // eod
};

void Imap::Mailbox::Model::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Model *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->alertReceived((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->networkPolicyOffline(); break;
        case 2: _t->networkPolicyExpensive(); break;
        case 3: _t->networkPolicyOnline(); break;
        case 4: _t->networkPolicyChanged(); break;
        case 5: _t->networkError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->imapError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->authRequested(); break;
        case 8: _t->authAttemptFailed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->needsSslDecision((*reinterpret_cast< const QList<QSslCertificate>(*)>(_a[1])),(*reinterpret_cast< const QList<QSslError>(*)>(_a[2]))); break;
        case 10: _t->requireStartTlsInFuture(); break;
        case 11: _t->messageCountPossiblyChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 12: _t->mailboxCreationSucceded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->mailboxCreationFailed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 14: _t->mailboxDeletionSucceded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->mailboxDeletionFailed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 16: _t->mailboxSyncFailed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 17: _t->connectionStateChanged((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< Imap::ConnectionState(*)>(_a[2]))); break;
        case 18: _t->logParserFatalError((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QByteArray(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 19: _t->mailboxSyncingProgress((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< Imap::Mailbox::MailboxSyncingProgress(*)>(_a[2]))); break;
        case 20: _t->mailboxFirstUnseenMessage((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 21: _t->threadingAvailable((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])),(*reinterpret_cast< const QVector<Imap::Responses::ThreadingNode>(*)>(_a[4]))); break;
        case 22: _t->threadingFailed((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3]))); break;
        case 23: _t->capabilitiesUpdated((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 24: _t->logged((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< const Common::LogMessage(*)>(_a[2]))); break;
        case 25: _t->imapAuthErrorChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 26: _t->reloadMailboxList(); break;
        case 27: _t->switchToMailbox((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 28: { QAbstractItemModel* _r = _t->taskModel();
            if (_a[0]) *reinterpret_cast< QAbstractItemModel**>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setSslPolicy((*reinterpret_cast< const QList<QSslCertificate>(*)>(_a[1])),(*reinterpret_cast< const QList<QSslError>(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 30: _t->invalidateAllMessageCounts(); break;
        case 31: { QString _r = _t->imapAuthError();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 32: _t->handleSocketStateChanged((*reinterpret_cast< Imap::Parser*(*)>(_a[1])),(*reinterpret_cast< Imap::ConnectionState(*)>(_a[2]))); break;
        case 33: _t->slotParserLineReceived((*reinterpret_cast< Imap::Parser*(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 34: _t->slotParserLineSent((*reinterpret_cast< Imap::Parser*(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 35: _t->slotTasksChanged(); break;
        case 36: _t->slotTaskDying((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 37: _t->setImapAuthError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 38: _t->responseReceived(); break;
        case 39: _t->responseReceived((*reinterpret_cast< Imap::Parser*(*)>(_a[1]))); break;
        case 40: _t->askForChildrenOfMailbox((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const Imap::Mailbox::CacheLoadingMode(*)>(_a[2]))); break;
        case 41: _t->askForMessagesInMailbox((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 42: _t->runReadyTasks(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<Imap::Responses::ThreadingNode> >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Parser* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Parser* >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Parser* >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Parser* >(); break;
            }
            break;
        case 40:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Mailbox::CacheLoadingMode >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Model::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::alertReceived)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Model::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::networkPolicyOffline)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Model::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::networkPolicyExpensive)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Model::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::networkPolicyOnline)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Model::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::networkPolicyChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::networkError)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::imapError)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Model::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::authRequested)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::authAttemptFailed)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QList<QSslCertificate> & , const QList<QSslError> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::needsSslDecision)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Model::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::requireStartTlsInFuture)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QModelIndex & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::messageCountPossiblyChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::mailboxCreationSucceded)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::mailboxCreationFailed)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::mailboxDeletionSucceded)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::mailboxDeletionFailed)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::mailboxSyncFailed)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (Model::*)(uint , Imap::ConnectionState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::connectionStateChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (Model::*)(uint , const QString & , const QString & , const QByteArray & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::logParserFatalError)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QModelIndex & , Imap::Mailbox::MailboxSyncingProgress );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::mailboxSyncingProgress)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QModelIndex & , const QModelIndex & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::mailboxFirstUnseenMessage)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QModelIndex & , const QByteArray & , const QStringList & , const QVector<Imap::Responses::ThreadingNode> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::threadingAvailable)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QModelIndex & , const QByteArray & , const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::threadingFailed)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::capabilitiesUpdated)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (Model::*)(uint , const Common::LogMessage & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::logged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (Model::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Model::imapAuthErrorChanged)) {
                *result = 25;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Model *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->imapUser(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->imapPassword(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->imapAuthError(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isNetworkAvailable(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isNetworkOnline(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Model *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setImapUser(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setImapPassword(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        Model *_t = static_cast<Model *>(_o);
        (void)_t;
        switch (_id) {
        case 1: _t->unsetImapPassword(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::Model::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__Model.data,
    qt_meta_data_Imap__Mailbox__Model,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::Model::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::Model::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__Model.stringdata0))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int Imap::Mailbox::Model::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Imap::Mailbox::Model::alertReceived(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Imap::Mailbox::Model::networkPolicyOffline()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Imap::Mailbox::Model::networkPolicyExpensive()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Imap::Mailbox::Model::networkPolicyOnline()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Imap::Mailbox::Model::networkPolicyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Imap::Mailbox::Model::networkError(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Imap::Mailbox::Model::imapError(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Imap::Mailbox::Model::authRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Imap::Mailbox::Model::authAttemptFailed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void Imap::Mailbox::Model::needsSslDecision(const QList<QSslCertificate> & _t1, const QList<QSslError> & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void Imap::Mailbox::Model::requireStartTlsInFuture()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Imap::Mailbox::Model::messageCountPossiblyChanged(const QModelIndex & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void Imap::Mailbox::Model::mailboxCreationSucceded(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void Imap::Mailbox::Model::mailboxCreationFailed(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void Imap::Mailbox::Model::mailboxDeletionSucceded(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void Imap::Mailbox::Model::mailboxDeletionFailed(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void Imap::Mailbox::Model::mailboxSyncFailed(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void Imap::Mailbox::Model::connectionStateChanged(uint _t1, Imap::ConnectionState _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void Imap::Mailbox::Model::logParserFatalError(uint _t1, const QString & _t2, const QString & _t3, const QByteArray & _t4, int _t5)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void Imap::Mailbox::Model::mailboxSyncingProgress(const QModelIndex & _t1, Imap::Mailbox::MailboxSyncingProgress _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void Imap::Mailbox::Model::mailboxFirstUnseenMessage(const QModelIndex & _t1, const QModelIndex & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void Imap::Mailbox::Model::threadingAvailable(const QModelIndex & _t1, const QByteArray & _t2, const QStringList & _t3, const QVector<Imap::Responses::ThreadingNode> & _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}

// SIGNAL 22
void Imap::Mailbox::Model::threadingFailed(const QModelIndex & _t1, const QByteArray & _t2, const QStringList & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 22, _a);
}

// SIGNAL 23
void Imap::Mailbox::Model::capabilitiesUpdated(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 23, _a);
}

// SIGNAL 24
void Imap::Mailbox::Model::logged(uint _t1, const Common::LogMessage & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 24, _a);
}

// SIGNAL 25
void Imap::Mailbox::Model::imapAuthErrorChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 25, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
