/****************************************************************************
** Meta object code from reading C++ file 'SimplePartWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/SimplePartWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SimplePartWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__SimplePartWidget_t {
    QByteArrayData data[15];
    char stringdata0[193];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__SimplePartWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__SimplePartWidget_t qt_meta_stringdata_Gui__SimplePartWidget = {
    {
QT_MOC_LITERAL(0, 0, 21), // "Gui::SimplePartWidget"
QT_MOC_LITERAL(1, 22, 11), // "linkHovered"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 4), // "link"
QT_MOC_LITERAL(4, 40, 5), // "title"
QT_MOC_LITERAL(5, 46, 11), // "textContent"
QT_MOC_LITERAL(6, 58, 21), // "slotFileNameRequested"
QT_MOC_LITERAL(7, 80, 8), // "QString*"
QT_MOC_LITERAL(8, 89, 8), // "fileName"
QT_MOC_LITERAL(9, 98, 19), // "slotMarkupPlainText"
QT_MOC_LITERAL(10, 118, 16), // "slotDownloadPart"
QT_MOC_LITERAL(11, 135, 19), // "slotDownloadMessage"
QT_MOC_LITERAL(12, 155, 17), // "slotDownloadImage"
QT_MOC_LITERAL(13, 173, 15), // "QNetworkRequest"
QT_MOC_LITERAL(14, 189, 3) // "req"

    },
    "Gui::SimplePartWidget\0linkHovered\0\0"
    "link\0title\0textContent\0slotFileNameRequested\0"
    "QString*\0fileName\0slotMarkupPlainText\0"
    "slotDownloadPart\0slotDownloadMessage\0"
    "slotDownloadImage\0QNetworkRequest\0req"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__SimplePartWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,   44,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   51,    2, 0x08 /* Private */,
       9,    0,   54,    2, 0x08 /* Private */,
      10,    0,   55,    2, 0x08 /* Private */,
      11,    0,   56,    2, 0x08 /* Private */,
      12,    1,   57,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,    3,    4,    5,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,

       0        // eod
};

void Gui::SimplePartWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SimplePartWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->linkHovered((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 1: _t->slotFileNameRequested((*reinterpret_cast< QString*(*)>(_a[1]))); break;
        case 2: _t->slotMarkupPlainText(); break;
        case 3: _t->slotDownloadPart(); break;
        case 4: _t->slotDownloadMessage(); break;
        case 5: _t->slotDownloadImage((*reinterpret_cast< const QNetworkRequest(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNetworkRequest >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SimplePartWidget::*)(const QString & , const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SimplePartWidget::linkHovered)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::SimplePartWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<EmbeddedWebView::staticMetaObject>(),
    qt_meta_stringdata_Gui__SimplePartWidget.data,
    qt_meta_data_Gui__SimplePartWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::SimplePartWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::SimplePartWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__SimplePartWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AbstractPartWidget"))
        return static_cast< AbstractPartWidget*>(this);
    return EmbeddedWebView::qt_metacast(_clname);
}

int Gui::SimplePartWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EmbeddedWebView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Gui::SimplePartWidget::linkHovered(const QString & _t1, const QString & _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
