/****************************************************************************
** Meta object code from reading C++ file 'AbookAddressbook.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Plugins/AbookAddressbook/AbookAddressbook.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AbookAddressbook.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AbookAddressbook_t {
    QByteArrayData data[18];
    char stringdata0[250];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AbookAddressbook_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AbookAddressbook_t qt_meta_stringdata_AbookAddressbook = {
    {
QT_MOC_LITERAL(0, 0, 16), // "AbookAddressbook"
QT_MOC_LITERAL(1, 17, 17), // "requestCompletion"
QT_MOC_LITERAL(2, 35, 25), // "AddressbookCompletionJob*"
QT_MOC_LITERAL(3, 61, 0), // ""
QT_MOC_LITERAL(4, 62, 5), // "input"
QT_MOC_LITERAL(5, 68, 7), // "ignores"
QT_MOC_LITERAL(6, 76, 3), // "max"
QT_MOC_LITERAL(7, 80, 28), // "requestPrettyNamesForAddress"
QT_MOC_LITERAL(8, 109, 20), // "AddressbookNamesJob*"
QT_MOC_LITERAL(9, 130, 5), // "email"
QT_MOC_LITERAL(10, 136, 21), // "openAddressbookWindow"
QT_MOC_LITERAL(11, 158, 17), // "openContactWindow"
QT_MOC_LITERAL(12, 176, 11), // "displayName"
QT_MOC_LITERAL(13, 188, 12), // "saveContacts"
QT_MOC_LITERAL(14, 201, 9), // "readAbook"
QT_MOC_LITERAL(15, 211, 6), // "update"
QT_MOC_LITERAL(16, 218, 11), // "updateAbook"
QT_MOC_LITERAL(17, 230, 19) // "scheduleAbookUpdate"

    },
    "AbookAddressbook\0requestCompletion\0"
    "AddressbookCompletionJob*\0\0input\0"
    "ignores\0max\0requestPrettyNamesForAddress\0"
    "AddressbookNamesJob*\0email\0"
    "openAddressbookWindow\0openContactWindow\0"
    "displayName\0saveContacts\0readAbook\0"
    "update\0updateAbook\0scheduleAbookUpdate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AbookAddressbook[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   69,    3, 0x0a /* Public */,
       1,    2,   76,    3, 0x2a /* Public | MethodCloned */,
       1,    1,   81,    3, 0x2a /* Public | MethodCloned */,
       7,    1,   84,    3, 0x0a /* Public */,
      10,    0,   87,    3, 0x0a /* Public */,
      11,    2,   88,    3, 0x0a /* Public */,
      13,    0,   93,    3, 0x0a /* Public */,
      14,    1,   94,    3, 0x0a /* Public */,
      14,    0,   97,    3, 0x2a /* Public | MethodCloned */,
      16,    0,   98,    3, 0x0a /* Public */,
      17,    0,   99,    3, 0x08 /* Private */,

 // slots: parameters
    0x80000000 | 2, QMetaType::QString, QMetaType::QStringList, QMetaType::Int,    4,    5,    6,
    0x80000000 | 2, QMetaType::QString, QMetaType::QStringList,    4,    5,
    0x80000000 | 2, QMetaType::QString,    4,
    0x80000000 | 8, QMetaType::QString,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    9,   12,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void AbookAddressbook::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AbookAddressbook *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { AddressbookCompletionJob* _r = _t->requestCompletion((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< AddressbookCompletionJob**>(_a[0]) = std::move(_r); }  break;
        case 1: { AddressbookCompletionJob* _r = _t->requestCompletion((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< AddressbookCompletionJob**>(_a[0]) = std::move(_r); }  break;
        case 2: { AddressbookCompletionJob* _r = _t->requestCompletion((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< AddressbookCompletionJob**>(_a[0]) = std::move(_r); }  break;
        case 3: { AddressbookNamesJob* _r = _t->requestPrettyNamesForAddress((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< AddressbookNamesJob**>(_a[0]) = std::move(_r); }  break;
        case 4: _t->openAddressbookWindow(); break;
        case 5: _t->openContactWindow((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->saveContacts(); break;
        case 7: _t->readAbook((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->readAbook(); break;
        case 9: _t->updateAbook(); break;
        case 10: _t->scheduleAbookUpdate(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AbookAddressbook::staticMetaObject = { {
    QMetaObject::SuperData::link<AddressbookPlugin::staticMetaObject>(),
    qt_meta_stringdata_AbookAddressbook.data,
    qt_meta_data_AbookAddressbook,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AbookAddressbook::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AbookAddressbook::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AbookAddressbook.stringdata0))
        return static_cast<void*>(this);
    return AddressbookPlugin::qt_metacast(_clname);
}

int AbookAddressbook::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AddressbookPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}
struct qt_meta_stringdata_trojita_plugin_AbookAddressbookPlugin_t {
    QByteArrayData data[1];
    char stringdata0[38];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_trojita_plugin_AbookAddressbookPlugin_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_trojita_plugin_AbookAddressbookPlugin_t qt_meta_stringdata_trojita_plugin_AbookAddressbookPlugin = {
    {
QT_MOC_LITERAL(0, 0, 37) // "trojita_plugin_AbookAddressbo..."

    },
    "trojita_plugin_AbookAddressbookPlugin"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_trojita_plugin_AbookAddressbookPlugin[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void trojita_plugin_AbookAddressbookPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject trojita_plugin_AbookAddressbookPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_trojita_plugin_AbookAddressbookPlugin.data,
    qt_meta_data_trojita_plugin_AbookAddressbookPlugin,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *trojita_plugin_AbookAddressbookPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *trojita_plugin_AbookAddressbookPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_trojita_plugin_AbookAddressbookPlugin.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AddressbookPluginInterface"))
        return static_cast< AddressbookPluginInterface*>(this);
    if (!strcmp(_clname, "AddressbookPluginInterface"))
        return static_cast< Plugins::AddressbookPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int trojita_plugin_AbookAddressbookPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x37,  'n',  'e',  't',  '.',  'f', 
    'l',  'a',  's',  'k',  'a',  '.',  't',  'r', 
    'o',  'j',  'i',  't',  'a',  '.',  'p',  'l', 
    'u',  'g',  'i',  'n',  's',  '.',  'a',  'd', 
    'd',  'r',  'e',  's',  's',  'b',  'o',  'o', 
    'k',  '.',  'a',  'b',  'o',  'o',  'k',  'a', 
    'd',  'd',  'r',  'e',  's',  's',  'b',  'o', 
    'o',  'k', 
    // "className"
    0x03,  0x78,  0x25,  't',  'r',  'o',  'j',  'i', 
    't',  'a',  '_',  'p',  'l',  'u',  'g',  'i', 
    'n',  '_',  'A',  'b',  'o',  'o',  'k',  'A', 
    'd',  'd',  'r',  'e',  's',  's',  'b',  'o', 
    'o',  'k',  'P',  'l',  'u',  'g',  'i',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(trojita_plugin_AbookAddressbookPlugin, trojita_plugin_AbookAddressbookPlugin)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
