/********************************************************************************
** Form generated from reading UI file 'AboutDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_AboutDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *main;
    QVBoxLayout *leftColumn;
    QLabel *iconLabel;
    QLabel *versionLabel;
    QLabel *wwwLabel;
    QSpacerItem *spacer1;
    QLabel *qtLogo;
    QLabel *qtVersion;
    QSpacerItem *spacer2;
    QVBoxLayout *rightColumn;
    QLabel *descriptionLabel;
    QScrollArea *scrollableCredits;
    QLabel *credits;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *AboutDialog)
    {
        if (AboutDialog->objectName().isEmpty())
            AboutDialog->setObjectName(QString::fromUtf8("AboutDialog"));
        AboutDialog->resize(623, 382);
        verticalLayout = new QVBoxLayout(AboutDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(-1, -1, 0, -1);
        main = new QHBoxLayout();
        main->setSpacing(10);
        main->setObjectName(QString::fromUtf8("main"));
        leftColumn = new QVBoxLayout();
        leftColumn->setObjectName(QString::fromUtf8("leftColumn"));
        iconLabel = new QLabel(AboutDialog);
        iconLabel->setObjectName(QString::fromUtf8("iconLabel"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(iconLabel->sizePolicy().hasHeightForWidth());
        iconLabel->setSizePolicy(sizePolicy);
        iconLabel->setMinimumSize(QSize(144, 128));
        iconLabel->setPixmap(QPixmap(QString::fromUtf8(":/icons/trojita-144.png")));
        iconLabel->setAlignment(Qt::AlignHCenter|Qt::AlignTop);

        leftColumn->addWidget(iconLabel);

        versionLabel = new QLabel(AboutDialog);
        versionLabel->setObjectName(QString::fromUtf8("versionLabel"));
        versionLabel->setText(QString::fromUtf8("VersionPlaceholder"));
        versionLabel->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
        versionLabel->setWordWrap(true);
        versionLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        leftColumn->addWidget(versionLabel);

        wwwLabel = new QLabel(AboutDialog);
        wwwLabel->setObjectName(QString::fromUtf8("wwwLabel"));
        wwwLabel->setText(QString::fromUtf8("<a href=\"http://trojita.flaska.net/\">http://trojita.flaska.net/</a>"));
        wwwLabel->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
        wwwLabel->setOpenExternalLinks(true);
        wwwLabel->setTextInteractionFlags(Qt::LinksAccessibleByKeyboard|Qt::LinksAccessibleByMouse);

        leftColumn->addWidget(wwwLabel);

        spacer1 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Fixed);

        leftColumn->addItem(spacer1);

        qtLogo = new QLabel(AboutDialog);
        qtLogo->setObjectName(QString::fromUtf8("qtLogo"));
        sizePolicy.setHeightForWidth(qtLogo->sizePolicy().hasHeightForWidth());
        qtLogo->setSizePolicy(sizePolicy);
        qtLogo->setMinimumSize(QSize(142, 34));
        qtLogo->setPixmap(QPixmap(QString::fromUtf8(":/icons/Built_with_Qt_logo_RGB.svg")));

        leftColumn->addWidget(qtLogo);

        qtVersion = new QLabel(AboutDialog);
        qtVersion->setObjectName(QString::fromUtf8("qtVersion"));
        qtVersion->setText(QString::fromUtf8("QtVersion"));
        qtVersion->setAlignment(Qt::AlignHCenter|Qt::AlignTop);

        leftColumn->addWidget(qtVersion);

        spacer2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        leftColumn->addItem(spacer2);


        main->addLayout(leftColumn);

        rightColumn = new QVBoxLayout();
        rightColumn->setSpacing(20);
        rightColumn->setObjectName(QString::fromUtf8("rightColumn"));
        descriptionLabel = new QLabel(AboutDialog);
        descriptionLabel->setObjectName(QString::fromUtf8("descriptionLabel"));
        descriptionLabel->setWordWrap(true);

        rightColumn->addWidget(descriptionLabel);

        scrollableCredits = new QScrollArea(AboutDialog);
        scrollableCredits->setObjectName(QString::fromUtf8("scrollableCredits"));
        scrollableCredits->setFrameShape(QFrame::NoFrame);
        scrollableCredits->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        scrollableCredits->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        scrollableCredits->setWidgetResizable(true);
        credits = new QLabel();
        credits->setObjectName(QString::fromUtf8("credits"));
        QFont font;
        font.setFamily(QString::fromUtf8("Monospace"));
        credits->setFont(font);
        credits->setText(QString::fromUtf8("LICENSE placeholder"));
        credits->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        credits->setWordWrap(true);
        credits->setProperty("readOnly", QVariant(true));
        scrollableCredits->setWidget(credits);

        rightColumn->addWidget(scrollableCredits);

        rightColumn->setStretch(1, 1);

        main->addLayout(rightColumn);


        verticalLayout->addLayout(main);

        buttonBox = new QDialogButtonBox(AboutDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(true);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(AboutDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), AboutDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), AboutDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *AboutDialog)
    {
        AboutDialog->setWindowTitle(QCoreApplication::translate("AboutDialog", "About Trojit\303\241", nullptr));
        descriptionLabel->setText(QCoreApplication::translate("AboutDialog", "<p>This is <b>Trojit\303\241</b>, a fast Qt IMAP e-mail client</p>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDIALOG_H
