/********************************************************************************
** Form generated from reading UI file 'SettingsGeneralPage.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGSGENERALPAGE_H
#define UI_SETTINGSGENERALPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_GeneralPage
{
public:
    QWidget *scrollAreaWidgetContents;
    QGridLayout *gridLayout;
    QCheckBox *showHomepageCheckbox;
    QCheckBox *preferPlaintextCheckbox;
    QCheckBox *guiSystrayCheckbox;
    QCheckBox *guiStartMinimizedCheckbox;
    QTableView *identityTabelView;
    QVBoxLayout *verticalLayout;
    QPushButton *moveUpButton;
    QPushButton *moveDownButton;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QPushButton *addButton;
    QPushButton *editButton;
    QPushButton *deleteButton;
    QFormLayout *formLayout;
    QLabel *addressbookLabel;
    QComboBox *addressbookBox;
    QLabel *passwordLabel;
    QComboBox *passwordBox;
    QLabel *spellcheckerLabel;
    QComboBox *spellcheckerBox;
    QFrame *line;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *markReadCheckbox;
    QSpinBox *markReadSeconds;
    QCheckBox *revealTrojitaVersions;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *mboxDropActionCheckbox;
    QComboBox *mboxDropActionBox;

    void setupUi(QScrollArea *GeneralPage)
    {
        if (GeneralPage->objectName().isEmpty())
            GeneralPage->setObjectName(QString::fromUtf8("GeneralPage"));
        GeneralPage->resize(417, 432);
        GeneralPage->setFrameShadow(QFrame::Plain);
        GeneralPage->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 415, 536));
        scrollAreaWidgetContents->setLayoutDirection(Qt::LeftToRight);
        gridLayout = new QGridLayout(scrollAreaWidgetContents);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        showHomepageCheckbox = new QCheckBox(scrollAreaWidgetContents);
        showHomepageCheckbox->setObjectName(QString::fromUtf8("showHomepageCheckbox"));

        gridLayout->addWidget(showHomepageCheckbox, 8, 0, 1, 2);

        preferPlaintextCheckbox = new QCheckBox(scrollAreaWidgetContents);
        preferPlaintextCheckbox->setObjectName(QString::fromUtf8("preferPlaintextCheckbox"));

        gridLayout->addWidget(preferPlaintextCheckbox, 9, 0, 1, 2);

        guiSystrayCheckbox = new QCheckBox(scrollAreaWidgetContents);
        guiSystrayCheckbox->setObjectName(QString::fromUtf8("guiSystrayCheckbox"));

        gridLayout->addWidget(guiSystrayCheckbox, 10, 0, 1, 2);

        guiStartMinimizedCheckbox = new QCheckBox(scrollAreaWidgetContents);
        guiStartMinimizedCheckbox->setObjectName(QString::fromUtf8("guiStartMinimizedCheckbox"));

        gridLayout->addWidget(guiStartMinimizedCheckbox, 11, 0, 1, 1);

        identityTabelView = new QTableView(scrollAreaWidgetContents);
        identityTabelView->setObjectName(QString::fromUtf8("identityTabelView"));
        identityTabelView->setMinimumSize(QSize(241, 192));

        gridLayout->addWidget(identityTabelView, 0, 0, 2, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        moveUpButton = new QPushButton(scrollAreaWidgetContents);
        moveUpButton->setObjectName(QString::fromUtf8("moveUpButton"));
        moveUpButton->setMinimumSize(QSize(29, 29));
        moveUpButton->setMaximumSize(QSize(29, 29));

        verticalLayout->addWidget(moveUpButton);

        moveDownButton = new QPushButton(scrollAreaWidgetContents);
        moveDownButton->setObjectName(QString::fromUtf8("moveDownButton"));
        moveDownButton->setMinimumSize(QSize(29, 29));
        moveDownButton->setMaximumSize(QSize(29, 29));

        verticalLayout->addWidget(moveDownButton);


        gridLayout->addLayout(verticalLayout, 0, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 125, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(verticalSpacer, 1, 1, 2, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetMinimumSize);
        addButton = new QPushButton(scrollAreaWidgetContents);
        addButton->setObjectName(QString::fromUtf8("addButton"));

        horizontalLayout->addWidget(addButton);

        editButton = new QPushButton(scrollAreaWidgetContents);
        editButton->setObjectName(QString::fromUtf8("editButton"));

        horizontalLayout->addWidget(editButton);

        deleteButton = new QPushButton(scrollAreaWidgetContents);
        deleteButton->setObjectName(QString::fromUtf8("deleteButton"));

        horizontalLayout->addWidget(deleteButton);


        gridLayout->addLayout(horizontalLayout, 2, 0, 1, 1);

        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        addressbookLabel = new QLabel(scrollAreaWidgetContents);
        addressbookLabel->setObjectName(QString::fromUtf8("addressbookLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, addressbookLabel);

        addressbookBox = new QComboBox(scrollAreaWidgetContents);
        addressbookBox->setObjectName(QString::fromUtf8("addressbookBox"));
        addressbookBox->setMinimumSize(QSize(310, 0));

        formLayout->setWidget(0, QFormLayout::FieldRole, addressbookBox);

        passwordLabel = new QLabel(scrollAreaWidgetContents);
        passwordLabel->setObjectName(QString::fromUtf8("passwordLabel"));

        formLayout->setWidget(1, QFormLayout::LabelRole, passwordLabel);

        passwordBox = new QComboBox(scrollAreaWidgetContents);
        passwordBox->setObjectName(QString::fromUtf8("passwordBox"));
        passwordBox->setMinimumSize(QSize(310, 0));

        formLayout->setWidget(1, QFormLayout::FieldRole, passwordBox);

        spellcheckerLabel = new QLabel(scrollAreaWidgetContents);
        spellcheckerLabel->setObjectName(QString::fromUtf8("spellcheckerLabel"));

        formLayout->setWidget(2, QFormLayout::LabelRole, spellcheckerLabel);

        spellcheckerBox = new QComboBox(scrollAreaWidgetContents);
        spellcheckerBox->setObjectName(QString::fromUtf8("spellcheckerBox"));

        formLayout->setWidget(2, QFormLayout::FieldRole, spellcheckerBox);


        gridLayout->addLayout(formLayout, 3, 0, 1, 2);

        line = new QFrame(scrollAreaWidgetContents);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        gridLayout->addWidget(line, 4, 0, 1, 2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(4);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        markReadCheckbox = new QCheckBox(scrollAreaWidgetContents);
        markReadCheckbox->setObjectName(QString::fromUtf8("markReadCheckbox"));

        horizontalLayout_2->addWidget(markReadCheckbox);

        markReadSeconds = new QSpinBox(scrollAreaWidgetContents);
        markReadSeconds->setObjectName(QString::fromUtf8("markReadSeconds"));
        markReadSeconds->setMinimum(0);
        markReadSeconds->setMaximum(60);
        markReadSeconds->setValue(0);

        horizontalLayout_2->addWidget(markReadSeconds);


        gridLayout->addLayout(horizontalLayout_2, 5, 0, 1, 1);

        revealTrojitaVersions = new QCheckBox(scrollAreaWidgetContents);
        revealTrojitaVersions->setObjectName(QString::fromUtf8("revealTrojitaVersions"));

        gridLayout->addWidget(revealTrojitaVersions, 7, 0, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        mboxDropActionCheckbox = new QCheckBox(scrollAreaWidgetContents);
        mboxDropActionCheckbox->setObjectName(QString::fromUtf8("mboxDropActionCheckbox"));
        mboxDropActionCheckbox->setEnabled(true);

        horizontalLayout_3->addWidget(mboxDropActionCheckbox);

        mboxDropActionBox = new QComboBox(scrollAreaWidgetContents);
        mboxDropActionBox->setObjectName(QString::fromUtf8("mboxDropActionBox"));
        mboxDropActionBox->setEnabled(false);

        horizontalLayout_3->addWidget(mboxDropActionBox);


        gridLayout->addLayout(horizontalLayout_3, 6, 0, 1, 1);

        GeneralPage->setWidget(scrollAreaWidgetContents);

        retranslateUi(GeneralPage);

        QMetaObject::connectSlotsByName(GeneralPage);
    } // setupUi

    void retranslateUi(QScrollArea *GeneralPage)
    {
        GeneralPage->setWindowTitle(QCoreApplication::translate("GeneralPage", "ScrollArea", nullptr));
#if QT_CONFIG(tooltip)
        showHomepageCheckbox->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        showHomepageCheckbox->setText(QCoreApplication::translate("GeneralPage", "Show Trojit\303\241's homepage on startup", nullptr));
        preferPlaintextCheckbox->setText(QCoreApplication::translate("GeneralPage", "Show plaintext e-mails instead of the HTML version", nullptr));
        guiSystrayCheckbox->setText(QCoreApplication::translate("GeneralPage", "Show system tray icon", nullptr));
        guiStartMinimizedCheckbox->setText(QCoreApplication::translate("GeneralPage", "Start minimized", nullptr));
        moveUpButton->setText(QString());
        moveDownButton->setText(QString());
        addButton->setText(QCoreApplication::translate("GeneralPage", "Add", nullptr));
        editButton->setText(QCoreApplication::translate("GeneralPage", "Edit", nullptr));
        deleteButton->setText(QCoreApplication::translate("GeneralPage", "Delete", nullptr));
        addressbookLabel->setText(QCoreApplication::translate("GeneralPage", "Addressbook", nullptr));
        passwordLabel->setText(QCoreApplication::translate("GeneralPage", "Passwords", nullptr));
        spellcheckerLabel->setText(QCoreApplication::translate("GeneralPage", "Spell checker", nullptr));
        markReadCheckbox->setText(QCoreApplication::translate("GeneralPage", "Automatically mark messages as read", nullptr));
        markReadSeconds->setSpecialValueText(QCoreApplication::translate("GeneralPage", "immediately", nullptr));
        markReadSeconds->setSuffix(QCoreApplication::translate("GeneralPage", " s", nullptr));
        markReadSeconds->setPrefix(QCoreApplication::translate("GeneralPage", "after ", nullptr));
#if QT_CONFIG(tooltip)
        revealTrojitaVersions->setToolTip(QCoreApplication::translate("GeneralPage", "<html><head/><body><p>Selecting this option will include version information about Trojit\303\241 and the underlying platform, including the Qt version and operating system details. Please consider leaving this option enabled so that developers are able to troubleshoot bugreports when something goes wrong.</p><p>The data are communicated via the IMAP's <code>ID</code> command and through the <code>User-Agent</code> header in outgoing e-mails, among other things.</p><p>Please note that even if you disable this option, the servers which Trojit\303\241 talks to and the people who read your e-mails will still be able to find out that you are using (some version of) Trojit\303\241.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        revealTrojitaVersions->setText(QCoreApplication::translate("GeneralPage", "Reveal Trojit\303\241's version and platform information", nullptr));
#if QT_CONFIG(tooltip)
        mboxDropActionCheckbox->setToolTip(QCoreApplication::translate("GeneralPage", "<p>If enabled, you can set the default drag and drop action for the mailbox tree.</p><p>If disabled, Trojit\303\241 will ask you every time. You can always override the action using modifier keys when dropping (control: copy, shift: move).</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        mboxDropActionCheckbox->setText(QCoreApplication::translate("GeneralPage", "Default drop action for the mbox tree", nullptr));
    } // retranslateUi

};

namespace Ui {
    class GeneralPage: public Ui_GeneralPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGSGENERALPAGE_H
